@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -29,7 +29,6 @@ def getChildReaper():
 
 
 def reaper_cleanup():
-    global singleton
     s = singleton
     if not s:
         return
@@ -41,8 +40,13 @@ def reaper_cleanup():
 
 
 class ProcInfo:
+    __slots__ = (
+        "pid", "name", "command", "ignore",
+        "forget", "dead", "returncode",
+         "callback", "process",
+         )
     def __repr__(self):
-        return "ProcInfo(%s)" % self.__dict__
+        return "ProcInfo(%s : %s)" % (self.pid, self.command)
 
     def get_info(self) -> dict:
         info = {
@@ -69,6 +73,7 @@ class ProcInfo:
 # WNOHANG is a tricky beast, see:
 # https://github.com/gevent/gevent/issues/622
 class ChildReaper:
+    __slots__ = ("_quit", "_proc_info")
     #note: the quit callback will fire only once!
     def __init__(self, quit_cb=None):
         log("ChildReaper(%s)", quit_cb)
@@ -108,7 +113,7 @@ class ChildReaper:
         procinfo.process = process
         procinfo.returncode = process.poll()
         procinfo.dead = procinfo.returncode is not None
-        log("add_process(%s, %s, %s, %s, %s) pid=%s", process, name, command, ignore, forget, pid)
+        log("add_process%s pid=%s", (process, name, command, ignore, forget, callback), pid)
         #could have died already:
         self._proc_info.append(procinfo)
         if procinfo.dead:
