@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2019-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -15,8 +15,5 @@ class Handler:
     def get_digest(self) -> str:
         return None
 
-    def handle(self, packet) -> bool:
-        if not self.client.password:
-            return False
-        self.client.send_challenge_reply(packet, self.client.password)
-        return True
+    def handle(self, challenge, digest, prompt) -> bool:
+        return self.client.password
