@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,8 +8,7 @@ import os.path
 import sys
 from time import monotonic
 
-from gi.repository import GLib
-from gi.repository import GObject
+from gi.repository import GLib, GObject  # @UnresolvedImport
 
 from xpra.util import (
     u, net_utf8, nonl, sorted_nicely, print_nested_dict, envint, flatten_dict, typedict,
@@ -25,7 +24,7 @@ from xpra.os_util import (
 from xpra.simple_stats import std_unit
 from xpra.client.client_base import XpraClientBase, EXTRA_TIMEOUT
 from xpra.exit_codes import (
-    EXIT_OK, EXIT_CONNECTION_LOST, EXIT_TIMEOUT, EXIT_INTERNAL_ERROR,
+    EXIT_OK, EXIT_CONNECTION_FAILED, EXIT_TIMEOUT, EXIT_INTERNAL_ERROR,
     EXIT_FAILURE, EXIT_UNSUPPORTED, EXIT_REMOTE_ERROR, EXIT_FILE_TOO_BIG,
     EXIT_IO_ERROR, EXIT_NO_DATA,
     )
@@ -139,11 +138,13 @@ class CommandConnectClient(GObjectXpraClient):
         self.printing = False
         self.command_timeout = None
         #don't bother with many of these things for one-off commands:
-        for x in ("ui_client", "wants_aliases", "wants_encodings",
-                  "wants_versions", "wants_features", "wants_sound", "windows",
-                  "webcam", "keyboard", "mouse", "network-state",
+        for x in ("ui_client",
+                  "wants_aliases", "wants_encodings", "wants_versions", "wants_features", "wants_sound",
+                  "windows", "webcam", "keyboard", "mouse", "network-state",
                   ):
             self.hello_extra[x] = False
+            #for newer versions, it is easier:
+            self.hello_extra["wants"] = []
 
     def setup_connection(self, conn):
         protocol = super().setup_connection(conn)
@@ -172,7 +173,7 @@ class CommandConnectClient(GObjectXpraClient):
             if len(packet)>1:
                 details = ": %s" % csv(packet[1:])
             log.warn("Connection failed%s", details)
-            self.quit(EXIT_CONNECTION_LOST)
+            self.quit(EXIT_CONNECTION_FAILED)
         else:
             self.quit(EXIT_OK)
 
@@ -260,7 +261,8 @@ class ScreenshotXpraClient(CommandConnectClient):
         assert bytestostr(encoding)=="png", "expected png screenshot data but got %s" % bytestostr(encoding)
         if not img_data:
             self.warn_and_quit(EXIT_OK,
-                               "screenshot is empty and has not been saved (maybe there are no windows or they are not currently shown)")
+                               "screenshot is empty and has not been saved"+
+                               " (maybe there are no windows or they are not currently shown)")
             return
         if self.screenshot_filename=="-":
             output = os.fdopen(sys.stdout.fileno(), "wb", closefd=False)
@@ -269,7 +271,7 @@ class ScreenshotXpraClient(CommandConnectClient):
         with output:
             output.write(img_data)
             output.flush()
-        self.warn_and_quit(EXIT_OK, "screenshot %sx%s saved to: %s" % (w, h, self.screenshot_filename))
+        self.warn_and_quit(EXIT_OK, f"screenshot {w}x{h} saved to: {self.screenshot_filename!r}")
 
     def init_packet_handlers(self):
         super().init_packet_handlers()
@@ -293,7 +295,7 @@ class InfoXpraClient(CommandConnectClient):
 
     def do_command(self, caps : typedict):
         def print_fn(s):
-            sys.stdout.write("%s\n" % (s,))
+            sys.stdout.write(f"{s}\n")
         if not caps:
             self.quit(EXIT_NO_DATA)
             return
@@ -319,7 +321,7 @@ class InfoXpraClient(CommandConnectClient):
                         return w
                     v = fixvalue(v)
                     k = fixvalue(k)
-                    print_fn("%s=%s" % (k, nonl(v)))
+                    print_fn(f"{k}={nonl(v)}")
             else:
                 print_nested_dict(caps, print_fn=print_fn)
         except OSError:
@@ -391,8 +393,9 @@ class MonitorXpraClient(SendCommandConnectClient):
 
     def __init__(self, opts):
         super().__init__(opts)
-        for x in ("wants_features", "wants_events", "event_request"):
-            self.hello_extra[x] = True
+        for x in ("features", "events", "request"):
+            self.hello_extra[f"wants_{x}"] = True
+            self.hello_extra.setdefault("wants", []).append(x)
         self.hello_extra["request"] = "event"
         self.hello_extra["info-namespace"] = True
 
@@ -671,7 +674,7 @@ class PrintClient(SendCommandConnectClient):
             #replace with filename proposed
             self.filename = command[2]
             #read file from stdin
-            with open(sys.stdin.fileno(), mode='rb', closefd=False) as stdin_binary:
+            with open(sys.stdin.fileno(), mode="rb", closefd=False) as stdin_binary:
                 self.file_data = stdin_binary.read()
             log("read %i bytes from stdin", len(self.file_data))
         else:
@@ -709,6 +712,7 @@ class PrintClient(SendCommandConnectClient):
 
     def make_hello(self):
         capabilities = super().make_hello()
+        capabilities.setdefault("wants", []).append("features")
         capabilities["wants_features"] = True   #so we know if printing is supported or not
         capabilities["print_request"] = True    #marker to skip full setup
         capabilities["request"] = "print"
