@@ -1,22 +1,21 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2014-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2014-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 import gi
-gi.require_version("Gtk", "3.0")
-gi.require_version("Pango", "1.0")
-gi.require_version("GdkPixbuf", "2.0")
-from gi.repository import Pango, Gtk
+gi.require_version("Gtk", "3.0")  # @UndefinedVariable
+gi.require_version("Pango", "1.0")  # @UndefinedVariable
+gi.require_version("GdkPixbuf", "2.0")  # @UndefinedVariable
+from gi.repository import Pango, Gtk  # @UnresolvedImport
 
 from xpra.gtk_common.gtk_util import (
     add_close_accel, scaled_image, get_icon_pixbuf,
     )
 from xpra.gtk_common.gobject_compat import register_os_signals
-from xpra.util import typedict
-from xpra.os_util import bytestostr
+from xpra.util import typedict, net_utf8
 from xpra.log import Logger, enable_debug_for
 
 log = Logger("exec")
@@ -34,10 +33,7 @@ def udict(d):
     t = typedict()
     for k, v in d.items():
         if isinstance(k, bytes):
-            try:
-                k = net_utf8(k)
-            except Exception:
-                k = bytestostr(k)
+            k = net_utf8(k)
         t[k] = v
     return t
 
@@ -62,15 +58,15 @@ class StartNewCommand:
         vbox.set_spacing(0)
 
         self.entry = None
-        if xdg_menu:
+        if self.xdg_menu:
             # or use menus if we have xdg data:
             hbox = Gtk.HBox(False, 20)
             vbox.add(hbox)
             hbox.add(Gtk.Label("Category:"))
             self.category_combo = Gtk.ComboBoxText()
             hbox.add(self.category_combo)
-            for name in sorted(xdg_menu.keys()):
-                self.category_combo.append_text(name.decode("utf-8"))
+            for name in sorted(self.xdg_menu.keys()):
+                self.category_combo.append_text(name)
             self.category_combo.set_active(0)
             self.category_combo.connect("changed", self.category_changed)
 
@@ -127,12 +123,12 @@ class StartNewCommand:
 
 
     def category_changed(self, *args):
-        category = self.category_combo.get_active_text().encode("utf-8")
+        category = self.category_combo.get_active_text()
         entries = udict(udict(self.xdg_menu.dictget(category, {})).dictget("Entries", {}))
         log("category_changed(%s) category=%s, entries=%s", args, category, entries)
         self.command_combo.get_model().clear()
         for name in entries.keys():
-            self.command_combo.append_text(name.decode("utf-8"))
+            self.command_combo.append_text(name)
         if entries:
             self.command_combo.set_active(0)
 
@@ -140,14 +136,14 @@ class StartNewCommand:
         if not self.entry:
             return
         category = self.category_combo.get_active_text()
-        entries = udict(udict(self.xdg_menu.dictget(category.encode("utf-8"), {})).dictget("Entries", {}))
+        entries = udict(udict(self.xdg_menu.dictget(category, {})).dictget("Entries", {}))
         command_name = self.command_combo.get_active_text()
         log("command_changed(%s) category=%s, entries=%s, command_name=%s", args, category, entries, command_name)
         command = ""
         if entries and command_name:
-            command_props = udict(udict(entries).dictget(command_name.encode("utf-8"), {}))
+            command_props = udict(udict(entries).dictget(command_name, {}))
             log("command properties=%s", command_props)
-            command = udict(command_props).strget(b"command", "")
+            command = udict(command_props).strget("command", "")
         self.entry.set_text(command)
 
 
@@ -213,5 +209,4 @@ def main(): # pragma: no cover
 
 
 if __name__ == "__main__":  # pragma: no cover
-    v = main()
-    sys.exit(v)
+    sys.exit(main())
