@@ -1,12 +1,12 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 #pylint: disable-msg=E1101
 
 from xpra.platform.gui import get_native_tray_classes, get_native_tray_menu_helper_class
-from xpra.os_util import bytestostr, strtobytes
-from xpra.util import nonl, envint, make_instance, CLIENT_EXIT, XPRA_APP_ID
+from xpra.os_util import bytestostr
+from xpra.util import envint, make_instance, CLIENT_EXIT, XPRA_APP_ID
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
 from xpra.log import Logger
 
@@ -15,30 +15,26 @@ log = Logger("tray")
 TRAY_DELAY = envint("XPRA_TRAY_DELAY", 0)
 
 
-"""
-Mixin for supporting our system tray
-(not forwarding other application's trays - that's handled in WindowClient)
-"""
 class TrayClient(StubClientMixin):
+    """
+    Mixin for supporting our system tray
+    (not forwarding other application's trays - that's handled in WindowClient)
+    """
 
     def __init__(self):
-        StubClientMixin.__init__(self)
+        super().__init__()
         #settings:
-        self.tray_enabled = False
-        self.delay_tray = False
         self.tray_icon = None
         #state:
         self.tray = None
         self.menu_helper = None
 
-    def init(self, opts, _extra_args=[]):
-        self.tray_enabled = opts.tray
-        self.delay_tray = opts.delay_tray
-        self.tray_icon = opts.tray_icon
-        if not self.tray_enabled:
+    def init(self, opts):
+        if not opts.tray:
             return
+        self.tray_icon = opts.tray_icon
         self.menu_helper = self.make_tray_menu_helper()
-        if self.delay_tray:
+        if opts.delay_tray:
             self.connect("first-ui-received", self.setup_xpra_tray)
         else:
             #show shortly after the main loop starts running:
@@ -110,7 +106,7 @@ class TrayClient(StubClientMixin):
             menu = self.menu_helper.build()
         tray = self.make_tray(XPRA_APP_ID, menu, self.get_tray_title(), tray_icon_filename,
                               xpra_tray_geometry, xpra_tray_click, xpra_tray_mouseover, xpra_tray_exit)
-        log("setup_xpra_tray(%s)=%s", tray_icon_filename, tray)
+        log("setup_xpra_tray(%s)=%s (%s)", tray_icon_filename, tray, type(tray))
         if tray:
             def reset_tray_title():
                 tray.set_tooltip(self.get_tray_title())
@@ -123,7 +119,7 @@ class TrayClient(StubClientMixin):
         log("make_tray%s tray classes=%s", args, tc)
         return make_instance(tc, self, *args)
 
-    def get_tray_title(self):
+    def get_tray_title(self) -> list:
         t = []
         if self.session_name or self.server_session_name:
             t.append(self.session_name or self.server_session_name)
@@ -131,9 +127,12 @@ class TrayClient(StubClientMixin):
         if p:
             conn = getattr(p, "_conn", None)
             if conn:
-                t.append(bytestostr(conn.target))
+                from xpra.net.bytestreams import pretty_socket
+                cinfo = conn.get_info()
+                endpoint = pretty_socket(cinfo.get("endpoint", conn.target)).split("?")[0]
+                t.append(endpoint)
         if not t:
-            t.insert(0, u"Xpra")
-        v = u"\n".join(t)
-        log("get_tray_title()=%s (items=%s)", nonl(v), tuple(strtobytes(x) for x in t))
+            t.insert(0, "Xpra")
+        v = "\n".join(str(x) for x in t)
+        log("get_tray_title()=%r (items=%s)", v, tuple(bytestostr(x) for x in t))
         return v
