@@ -684,13 +684,9 @@ class WindowClient(StubClientMixin):
                 self, width, height, coding, len(data), ICON_SHRINKAGE, ICON_OVERLAY)
         if coding=="default":
             img = self.overlay_image
-        elif coding == "BGRA":
+        elif coding in ("BGRA", "RGBA"):
             rowstride = width*4
-            img = Image.frombytes("RGBA", (width,height), memoryview_to_bytes(data), "raw", "BGRA", rowstride, 1)
-            has_alpha = True
-        elif coding in ("BGRA", ):
-            rowstride = width*4
-            img = Image.frombytes("RGBA", (width,height), memoryview_to_bytes(data), "raw", "BGRA", rowstride, 1)
+            img = Image.frombytes("RGBA", (width,height), memoryview_to_bytes(data), "raw", coding, rowstride, 1)
             has_alpha = True
         else:
             from xpra.codecs.pillow.decoder import open_only
@@ -1288,6 +1284,9 @@ class WindowClient(StubClientMixin):
         focuslog("send_focus(%s)", wid)
         self.send("focus", wid, self.get_current_modifiers())
 
+    def has_focus(self, wid):
+        return self._focused and self._focused==wid
+
     def update_focus(self, wid, gotit):
         focused = self._focused
         focuslog(f"update_focus({wid}, {gotit}) focused={focused}, grabbed={self._window_with_grab}")
