@@ -1,31 +1,35 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013 - 2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 from threading import Lock
-from xpra.log import Logger
-log = Logger("codec", "video")
 
-from xpra.codecs.loader import get_codec, get_codec_error
+from xpra.codecs.loader import load_codec, get_codec, get_codec_error
 from xpra.util import csv, engs
+from xpra.log import Logger
 
+log = Logger("codec", "video")
 
 #the codec loader uses the names...
 #but we need the module name to be able to probe without loading the codec:
 CODEC_TO_MODULE = {
-                   "vpx"        : ["vpx"],
-                   "x264"       : ["enc_x264"],
-                   "x265"       : ["enc_x265"],
-                   "nvenc"      : ["nvenc"],
-                   "swscale"    : ["csc_swscale"],
-                   "libyuv"     : ["csc_libyuv"],
-                   "avcodec2"   : ["dec_avcodec2"],
-                   "ffmpeg"     : ["enc_ffmpeg"],
-                   }
+    "vpx"        : "vpx",
+    "x264"       : "enc_x264",
+    "x265"       : "enc_x265",
+    "nvenc"      : "nvenc",
+    "swscale"    : "csc_swscale",
+    "cython"     : "csc_cython",
+    "libyuv"     : "csc_libyuv",
+    "avcodec2"   : "dec_avcodec2",
+    "ffmpeg"     : "enc_ffmpeg",
+    "jpeg"       : "jpeg",
+    "webp"       : "webp",
+    "nvjpeg"     : "nvjpeg",
+    }
 
 def has_codec_module(module_name):
     top_module = "xpra.codecs.%s" % module_name
@@ -37,25 +41,24 @@ def has_codec_module(module_name):
         log("codec module %s cannot be loaded: %s", module_name, e)
         return False
 
-def try_import_modules(codec_names):
+def try_import_modules(*codec_names):
     names = []
     for codec_name in codec_names:
-        module_names = CODEC_TO_MODULE[codec_name]
-        for module_name in module_names:
-            if has_codec_module(module_name):
-                names.append(codec_name)
-                break
+        module_name = CODEC_TO_MODULE[codec_name]
+        if has_codec_module(module_name):
+            names.append(codec_name)
     return names
 
 #all the codecs we know about:
 #try to import the module that contains them (cheap check):
-ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265", "nvenc", "ffmpeg"])
-ALL_CSC_MODULE_OPTIONS = try_import_modules(["swscale", "libyuv"])
+ALL_VIDEO_ENCODER_OPTIONS = try_import_modules("x264", "vpx", "x265", "nvenc", "ffmpeg", "nvjpeg", "jpeg", "webp")
+HARDWARE_ENCODER_OPTIONS = try_import_modules("nvenc", "nvjpeg")
+ALL_CSC_MODULE_OPTIONS = try_import_modules("swscale", "cython", "libyuv")
 NO_GFX_CSC_OPTIONS = []
-ALL_VIDEO_DECODER_OPTIONS = try_import_modules(["avcodec2", "vpx"])
+ALL_VIDEO_DECODER_OPTIONS = try_import_modules("avcodec2", "vpx")
 
-PREFERRED_ENCODER_ORDER = ["nvenc", "x264", "vpx", "x265"]
-PREFERRED_DECODER_ORDER = ["avcodec2", "vpx"]
+PREFERRED_ENCODER_ORDER = ("nvenc", "nvjpeg", "x264", "vpx", "jpeg", "webp", "x265")
+PREFERRED_DECODER_ORDER = ("avcodec2", "vpx")
 log("video_helper: ALL_VIDEO_ENCODER_OPTIONS=%s", ALL_VIDEO_ENCODER_OPTIONS)
 log("video_helper: ALL_CSC_MODULE_OPTIONS=%s", ALL_CSC_MODULE_OPTIONS)
 log("video_helper: NO_GFX_CSC_OPTIONS=%s", NO_GFX_CSC_OPTIONS)
@@ -65,15 +68,13 @@ log("video_helper: ALL_VIDEO_DECODER_OPTIONS=%s", ALL_VIDEO_DECODER_OPTIONS)
 #on top of that, there are compatibility problems with gtk at times: OpenCL AMD and TLS don't mix well
 
 
-def get_encoder_module_names(x):
-    if x=="nvenc":
-        return ["nvenc"]
-    elif x.find("enc")>=0:
-        return [x]              #ie: "nvenc" or "enc_vpx"
-    return ["enc_"+x]           #ie: "enc_x264"
+def get_encoder_module_name(x):
+    if x.find("enc")>=0:
+        return x                #ie: "nvenc" or "enc_vpx"
+    return "enc_"+x             #ie: "enc_x264"
 
 def get_decoder_module_name(x):
-        return "dec_"+x         #ie: "dec_vpx"
+    return "dec_"+x         #ie: "dec_vpx"
 
 def get_csc_module_name(x):
     return "csc_"+x             #ie: "csc_swscale"
@@ -84,12 +85,11 @@ def get_DEFAULT_VIDEO_ENCODERS():
     """ returns all the video encoders installed """
     encoders = []
     for x in tuple(ALL_VIDEO_ENCODER_OPTIONS):
-        mods = get_encoder_module_names(x)
-        for mod in mods:
-            c = get_codec(mod)
-            if c:
-                encoders.append(x)
-                break
+        mod = get_encoder_module_name(x)
+        c = get_codec(mod)
+        if c:
+            encoders.append(x)
+            break
     return encoders
 
 def get_DEFAULT_CSC_MODULES():
@@ -113,7 +113,7 @@ def get_DEFAULT_VIDEO_DECODERS():
     return decoders
 
 
-class VideoHelper(object):
+class VideoHelper:
     """
         This class is a bit like a registry of known encoders, csc modules and decoders.
         The main instance, obtained by calling getVideoHelper, can be initialized
@@ -121,10 +121,10 @@ class VideoHelper(object):
         We can also clone it to modify it (used by per client proxy encoders)
     """
 
-    def __init__(self, vencspecs={}, cscspecs={}, vdecspecs={}, init=False):
-        self._video_encoder_specs = vencspecs
-        self._csc_encoder_specs = cscspecs
-        self._video_decoder_specs = vdecspecs
+    def __init__(self, vencspecs=None, cscspecs=None, vdecspecs=None, init=False):
+        self._video_encoder_specs = vencspecs or {}
+        self._csc_encoder_specs = cscspecs or {}
+        self._video_decoder_specs = vdecspecs or {}
         self.video_encoders = []
         self.csc_modules = []
         self.video_decoders = []
@@ -136,30 +136,39 @@ class VideoHelper(object):
         self._initialized = init
         self._lock = Lock()
 
-    def set_modules(self, video_encoders=[], csc_modules=[], video_decoders=[]):
+    def set_modules(self, video_encoders=(), csc_modules=(), video_decoders=()):
+        log("set_modules%s", (video_encoders, csc_modules, video_decoders))
         assert not self._initialized, "too late to set modules, the helper is already initialized!"
         def filt(name, inlist, all_list):
-            notfound = [x for x in inlist if x and x not in all_list]
-            if notfound:
-                log.warn("ignoring unknown %s: %s", name, csv(notfound))
-            return [x for x in inlist if x in all_list]
+            exclist = list(x[1:] for x in inlist if x and x.startswith("-"))
+            inclist = list(x for x in inlist if x and not x.startswith("-"))
+            if "all" in inclist:
+                inclist = all_list
+            else:
+                notfound = tuple(x for x in (exclist+inclist) if x and x not in all_list)
+                if notfound:
+                    log.warn("Warning: ignoring unknown %s: %s", name, csv(notfound))
+            return tuple(x for x in inclist if x not in exclist)
         self.video_encoders = filt("video encoders" , video_encoders,   ALL_VIDEO_ENCODER_OPTIONS)
         self.csc_modules    = filt("csc modules"    , csc_modules,      ALL_CSC_MODULE_OPTIONS)
         self.video_decoders = filt("video decoders" , video_decoders,   ALL_VIDEO_DECODER_OPTIONS)
-        log("VideoHelper.set_modules(%s, %s, %s) video encoders=%s, csc=%s, video decoders=%s",
-            csv(video_encoders), csv(csc_modules), csv(video_decoders), csv(self.video_encoders), csv(self.csc_modules), csv(self.video_decoders))
+        log("VideoHelper.set_modules(%r, %r, %r) video encoders=%s, csc=%s, video decoders=%s",
+            csv(video_encoders), csv(csc_modules), csv(video_decoders),
+            csv(self.video_encoders), csv(self.csc_modules), csv(self.video_decoders))
 
     def cleanup(self):
         with self._lock:
             #check again with lock held (in case of race):
             if not self._initialized:
                 return
-            for module in self._cleanup_modules:
+            cmods = self._cleanup_modules
+            self._cleanup_modules = []
+            log("VideoHelper.cleanup() cleanup modules=%s", cmods)
+            for module in cmods:
                 try:
                     module.cleanup_module()
-                except:
-                    log.error("error cleaning up %s", module, exc_info=True)
-            self._cleanup_modules = []
+                except Exception:
+                    log.error("Error cleaning up %s", module, exc_info=True)
             self._video_encoder_specs = {}
             self._csc_encoder_specs = {}
             self._video_decoder_specs = {}
@@ -185,7 +194,7 @@ class VideoHelper(object):
         vds = deepish_clone_dict(self._video_decoder_specs)
         return VideoHelper(ves, ces, vds, True)
 
-    def get_info(self):
+    def get_info(self) -> dict:
         d = {}
         einfo = d.setdefault("encoding", {})
         dinfo = d.setdefault("decoding", {})
@@ -194,22 +203,21 @@ class VideoHelper(object):
             for in_csc, specs in encoder_specs.items():
                 for spec in specs:
                     einfo.setdefault("%s_to_%s" % (in_csc, encoding), []).append(spec.codec_type)
-        for in_csc, specs in self._csc_encoder_specs.items():
-            for out_csc, specs in specs.items():
+        for in_csc, out_specs in self._csc_encoder_specs.items():
+            for out_csc, specs in out_specs.items():
                 cinfo["%s_to_%s" % (in_csc, out_csc)] = [spec.codec_type for spec in specs]
         for encoding, decoder_specs in self._video_decoder_specs.items():
             for out_csc, decoders in decoder_specs.items():
                 for decoder in decoders:
-                    decoder_name, _ = decoder
+                    decoder_name = decoder[0]
                     dinfo.setdefault("%s_to_%s" % (encoding, out_csc), []).append(decoder_name)
         def modstatus(x, def_list, active_list):
             #the module is present
             if x in active_list:
                 return "active"
-            elif x in def_list:
+            if x in def_list:
                 return "disabled"
-            else:
-                return "not found"
+            return "not found"
         venc = einfo.setdefault("video-encoder", {})
         for x in ALL_VIDEO_ENCODER_OPTIONS:
             venc["%s" % x] = modstatus(x, get_DEFAULT_VIDEO_ENCODERS(), self.video_encoders)
@@ -256,21 +264,22 @@ class VideoHelper(object):
         log(" will try video encoders: %s", csv(self.video_encoders))
         for x in self.video_encoders:
             try:
-                mods = get_encoder_module_names(x)
-                log(" modules for %s: %s", x, csv(mods))
-                for mod in mods:
-                    try:
-                        self.init_video_encoder_option(mod)
-                        break
-                    except Exception as e:
-                        log(" init_video_encoder_option(%s) error", mod, exc_info=True)
-                        log.warn("Warning: cannot load %s video encoder:", mod)
-                        log.warn(" %s", e)
-                        del e
+                mod = get_encoder_module_name(x)
+                load_codec(mod)
+                log(" encoder for %s: %s", x, mod)
+                try:
+                    self.init_video_encoder_option(mod)
+                except Exception as e:
+                    log(" init_video_encoder_option(%s) error", mod, exc_info=True)
+                    log.warn("Warning: cannot load %s video encoder:", mod)
+                    log.warn(" %s", e)
+                    del e
             except Exception as e:
+                log("error on %s", x, exc_info=True)
                 log.warn("Warning: cannot add %s encoder: %s", x, e)
                 del e
-        log("found %i video encoder%s: %s", len(self._video_encoder_specs), engs(self._video_encoder_specs), csv(self._video_encoder_specs))
+        log("found %i video encoder%s: %s",
+            len(self._video_encoder_specs), engs(self._video_encoder_specs), csv(self._video_encoder_specs))
 
     def init_video_encoder_option(self, encoder_name):
         encoder_module = get_codec(encoder_name)
@@ -281,20 +290,15 @@ class VideoHelper(object):
             log(" %s", get_codec_error(encoder_name))
             return
         encoder_type = encoder_module.get_type()
-        try:
-            encoder_module.init_module()
-            self._cleanup_modules.append(encoder_module)
-        except Exception as e:
-            log(" exception in %s module %s initialization %s: %s", encoder_type, encoder_module.__name__, encoder_module.init_module, e, exc_info=True)
-            raise
         encodings = encoder_module.get_encodings()
-        log(" %s encodings=%s", encoder_type, csv(encodings))
+        log(" %12s encodings=%s", encoder_type, csv(encodings))
         for encoding in encodings:
             colorspaces = encoder_module.get_input_colorspaces(encoding)
-            log(" %s input colorspaces for %s: %s", encoder_type, encoding, csv(colorspaces))
+            log(" %9s  input colorspaces for %5s: %s", encoder_type, encoding, csv(colorspaces))
             for colorspace in colorspaces:
                 spec = encoder_module.get_spec(encoding, colorspace)
                 self.add_encoder_spec(encoding, colorspace, spec)
+        log("video encoder options: %s", self._video_encoder_specs)
 
     def add_encoder_spec(self, encoding, colorspace, spec):
         self._video_encoder_specs.setdefault(encoding, {}).setdefault(colorspace, []).append(spec)
@@ -306,14 +310,16 @@ class VideoHelper(object):
         for x in self.csc_modules:
             try:
                 mod = get_csc_module_name(x)
+                load_codec(mod)
                 self.init_csc_option(mod)
-            except:
+            except Exception:
                 log.warn("init_csc_options() cannot add %s csc", x, exc_info=True)
         log(" csc specs: %s", csv(self._csc_encoder_specs))
         for src_format, d in sorted(self._csc_encoder_specs.items()):
             log(" %s - %s options:", src_format, len(d))
             for dst_format, specs in sorted(d.items()):
-                log("  * %s via: %s", dst_format, csv(sorted(spec.codec_type for spec in specs)))
+                log("  * %7s via: %s", dst_format, csv(sorted(spec.codec_type for spec in specs)))
+        log("csc options: %s", self._csc_encoder_specs)
 
     def init_csc_option(self, csc_name):
         csc_module = get_codec(csc_name)
@@ -323,18 +329,10 @@ class VideoHelper(object):
             log(" csc module %s could not be loaded:", csc_name)
             log(" %s", get_codec_error(csc_name))
             return
-        csc_type = csc_module.get_type()
-        try:
-            csc_module.init_module()
-            self._cleanup_modules.append(csc_module)
-        except Exception as e:
-            log("exception in %s module initialization %s: %s", csc_type, csc_module.init_module, e, exc_info=True)
-            log.warn("Warning: cannot use %s module %s: %s", csc_type, csc_module, e)
-            return
         in_cscs = csc_module.get_input_colorspaces()
         for in_csc in in_cscs:
             out_cscs = csc_module.get_output_colorspaces(in_csc)
-            log("%s output colorspaces for %s: %s", csc_module.get_type(), in_csc, csv(out_cscs))
+            log("%9s output colorspaces for %7s: %s", csc_module.get_type(), in_csc, csv(out_cscs))
             for out_csc in out_cscs:
                 spec = csc_module.get_spec(in_csc, out_csc)
                 self.add_csc_spec(in_csc, out_csc, spec)
@@ -349,10 +347,13 @@ class VideoHelper(object):
         for x in self.video_decoders:
             try:
                 mod = get_decoder_module_name(x)
+                load_codec(mod)
                 self.init_video_decoder_option(mod)
-            except:
+            except Exception:
                 log.warn("Warning: cannot add %s decoder", x, exc_info=True)
-        log("found %s video decoder%s: %s", len(self._video_decoder_specs), engs(self._video_decoder_specs), csv(self._video_decoder_specs))
+        log("found %s video decoder%s: %s",
+            len(self._video_decoder_specs), engs(self._video_decoder_specs), csv(self._video_decoder_specs))
+        log("video decoder options: %s", self._video_decoder_specs)
 
     def init_video_decoder_option(self, decoder_name):
         decoder_module = get_codec(decoder_name)
@@ -363,13 +364,6 @@ class VideoHelper(object):
             log(" %s", get_codec_error(decoder_name))
             return
         decoder_type = decoder_module.get_type()
-        try:
-            decoder_module.init_module()
-            self._cleanup_modules.append(decoder_module)
-        except Exception as e:
-            log("exception in %s module initialization %s: %s", decoder_type, decoder_module.init_module, e, exc_info=True)
-            log.warn("Warning: cannot use %s module %s: %s", decoder_type, decoder_module, e, exc_info=True)
-            return
         encodings = decoder_module.get_encodings()
         log(" %s encodings=%s", decoder_type, csv(encodings))
         for encoding in encodings:
@@ -377,7 +371,7 @@ class VideoHelper(object):
             log(" %s input colorspaces for %s: %s", decoder_type, encoding, csv(colorspaces))
             for colorspace in colorspaces:
                 output_colorspace = decoder_module.get_output_colorspace(encoding, colorspace)
-                log(" %s output colorspace for %s/%s: %s", decoder_type, encoding, colorspace, output_colorspace)
+                log(" %s output colorspace for %5s/%7s: %s", decoder_type, encoding, colorspace, output_colorspace)
                 try:
                     assert decoder_module.Decoder
                     self.add_decoder(encoding, colorspace, decoder_name, decoder_module)
@@ -386,7 +380,11 @@ class VideoHelper(object):
                     del e
 
     def add_decoder(self, encoding, colorspace, decoder_name, decoder_module):
-        self._video_decoder_specs.setdefault(encoding, {}).setdefault(colorspace, []).append((decoder_name, decoder_module))
+        self._video_decoder_specs.setdefault(
+            encoding, {}).setdefault(
+                colorspace, []).append(
+                    (decoder_name, decoder_module)
+                    )
 
 
     def get_server_full_csc_modes(self, *client_supported_csc_modes):
@@ -394,19 +392,22 @@ class VideoHelper(object):
             returns the CSC modes per encoding that the server can encode with.
             (taking into account the decoder's actual output colorspace for each encoding)
         """
+        log("get_server_full_csc_modes(%s) decoder encodings=%s",
+            client_supported_csc_modes, self._video_decoder_specs.keys())
         full_csc_modes = {}
         for encoding, encoding_specs in self._video_decoder_specs.items():
             assert encoding_specs is not None
             for colorspace, decoder_specs in sorted(encoding_specs.items()):
                 for decoder_name, decoder_module in decoder_specs:
-                    log("found decoder %s for %s with %s mode", decoder_name, encoding, colorspace)
                     #figure out the actual output colorspace:
                     output_colorspace = decoder_module.get_output_colorspace(encoding, colorspace)
+                    log("found decoder %12s for %5s with %7s mode, outputs '%s'",
+                             decoder_name, encoding, colorspace, output_colorspace)
                     if output_colorspace in client_supported_csc_modes:
                         encoding_colorspaces = full_csc_modes.setdefault(encoding, [])
                         if colorspace not in encoding_colorspaces:
                             encoding_colorspaces.append(colorspace)
-        log("get_client_full_csc_modes(%s)=%s", client_supported_csc_modes, full_csc_modes)
+        log("get_server_full_csc_modes(%s)=%s", client_supported_csc_modes, full_csc_modes)
         return full_csc_modes
 
 
@@ -419,7 +420,7 @@ class VideoHelper(object):
         supported_csc_modes = list(target_rgb_modes)
         for src_format, specs in self._csc_encoder_specs.items():
             for dst_format, csc_specs in specs.items():
-                if dst_format in target_rgb_modes and len(csc_specs)>0:
+                if dst_format in target_rgb_modes and csc_specs:
                     supported_csc_modes.append(src_format)
                     break
         supported_csc_modes = sorted(supported_csc_modes)
@@ -435,7 +436,7 @@ def getVideoHelper():
 
 
 def main():
-    from xpra.codecs.loader import log as loader_log, load_codecs
+    from xpra.codecs.loader import log as loader_log, load_codecs, show_codecs
     from xpra.util import print_nested_dict
     from xpra.log import enable_color
     from xpra.platform import program_context
@@ -445,6 +446,7 @@ def main():
             loader_log.enable_debug()
             log.enable_debug()
         load_codecs()
+        show_codecs()
         vh = getVideoHelper()
         vh.set_modules(ALL_VIDEO_ENCODER_OPTIONS, ALL_CSC_MODULE_OPTIONS, ALL_VIDEO_DECODER_OPTIONS)
         vh.init()
