@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -14,8 +14,8 @@ gi.require_version("Pango", "1.0")
 gi.require_version("GdkPixbuf", "2.0")
 from gi.repository import GLib, GdkPixbuf, Pango, GObject, Gtk, Gdk     #@UnresolvedImport
 
-from xpra.util import first_time, envint, envbool
-from xpra.os_util import strtobytes, WIN32, OSX
+from xpra.util import first_time, envint, envbool, print_nested_dict
+from xpra.os_util import strtobytes, WIN32, OSX, POSIX, is_X11
 from xpra.log import Logger
 
 log = Logger("gtk", "util")
@@ -25,7 +25,6 @@ alphalog = Logger("gtk", "alpha")
 SHOW_ALL_VISUALS = False
 #try to get workarea from GTK:
 GTK_WORKAREA = envbool("XPRA_GTK_WORKAREA", True)
-SMOOTH_SCROLL = envbool("XPRA_SMOOTH_SCROLL", True)
 
 GTK_VERSION_INFO = {}
 def get_gtk_version_info() -> dict:
@@ -200,8 +199,6 @@ em = Gdk.EventMask
 WINDOW_EVENT_MASK = em.STRUCTURE_MASK | em.KEY_PRESS_MASK | em.KEY_RELEASE_MASK \
         | em.POINTER_MOTION_MASK | em.BUTTON_PRESS_MASK | em.BUTTON_RELEASE_MASK \
         | em.PROPERTY_CHANGE_MASK | em.SCROLL_MASK
-if SMOOTH_SCROLL:
-    WINDOW_EVENT_MASK |= em.SMOOTH_SCROLL_MASK
 
 del em
 
@@ -237,9 +234,9 @@ BYTE_ORDER_NAMES = {
 def get_screens_info() -> dict:
     display = Gdk.Display.get_default()
     info = {}
-    for i in range(display.get_n_screens()):
-        screen = display.get_screen(i)
-        info[i] = get_screen_info(display, screen)
+    assert display.get_n_screens()==1, "GTK3: The number of screens is always 1"
+    screen = display.get_screen(0)
+    info[0] = get_screen_info(display, screen)
     return info
 
 def get_screen_sizes(xscale=1, yscale=1):
@@ -272,9 +269,9 @@ def get_screen_sizes(xscale=1, yscale=1):
         monitor = display.get_monitor(j)
         geom = monitor.get_geometry()
         manufacturer, model = monitor.get_manufacturer(), monitor.get_model()
-        if manufacturer=="unknown":
+        if manufacturer in ("unknown", None):
             manufacturer = ""
-        if model=="unknown":
+        if model in ("unknown", None):
             model = ""
         if manufacturer and model:
             plug_name = "%s %s" % (manufacturer, model)
@@ -288,26 +285,41 @@ def get_screen_sizes(xscale=1, yscale=1):
         monitor_info = [plug_name, xs(geom.x), ys(geom.y), xs(geom.width), ys(geom.height), wmm, hmm]
         screenlog(" monitor %s: %s, model=%s, manufacturer=%s",
                   j, type(monitor).__name__, monitor.get_model(), monitor.get_manufacturer())
+        def vmwx(v):
+            return v<geom.x or v>geom.x+geom.width
+        def vmwy(v):
+            return v<geom.y or v>geom.y+geom.height
+        def valid_workarea(work_x, work_y, work_width, work_height):
+            if vmwx(work_x) or vmwx(work_x+work_width) or vmwy(work_y) or vmwy(work_y+work_height):
+                log("discarding invalid workarea: %s", (work_x, work_y, work_width, work_height))
+                return []
+            return list(swork(work_x, work_y, work_width, work_height))
         if GTK_WORKAREA and hasattr(monitor, "get_workarea"):
             rect = monitor.get_workarea()
-            monitor_info += list(swork(rect.x, rect.y, rect.width, rect.height))
+            monitor_info += valid_workarea(rect.x, rect.y, rect.width, rect.height)
         elif workareas:
-            w = workareas[j]
-            monitor_info += list(swork(*w))
+            monitor_info += valid_workarea(*workareas[j])
         monitors.append(tuple(monitor_info))
     screen = display.get_default_screen()
     sw, sh = screen.get_width(), screen.get_height()
     work_x, work_y, work_width, work_height = swork(0, 0, sw, sh)
     workarea = get_workarea()   #pylint: disable=assignment-from-none
+    screenlog(" workarea=%s", workarea)
     if workarea:
         work_x, work_y, work_width, work_height = swork(*workarea)  #pylint: disable=not-an-iterable
-    screenlog(" workarea=%s", workarea)
+        def vwx(v):
+            return v<0 or v>sw
+        def vwy(v):
+            return v<0 or v>sh
+        if vwx(work_x) or vwx(work_x+work_width) or vwy(work_y) or vwy(work_y+work_height):
+            screenlog(" discarding invalid workarea values: %s", workarea)
+            work_x, work_y, work_width, work_height = swork(0, 0, sw, sh)
     wmm = screen.get_width_mm()
     hmm = screen.get_height_mm()
     xdpi = dpi(sw, wmm)
     ydpi = dpi(sh, hmm)
     if xdpi<MIN_DPI or xdpi>MAX_DPI or ydpi<MIN_DPI or ydpi>MAX_DPI:
-        log("ignoring invalid screen size %ix%imm", wmm, hmm)
+        log(f"ignoring invalid DPI {xdpi},{ydpi} from screen size {wmm}x{hmm}mm")
         if os.environ.get("WAYLAND_DISPLAY"):
             log(" (wayland display?)")
         if n_monitors>0:
@@ -350,10 +362,10 @@ def get_screen_info(display, screen) -> dict:
             info[x] = int(fn())
         except Exception:
             pass
-    info["monitors"] = screen.get_n_monitors()
+    info["monitors"] = display.get_n_monitors()
     m_info = info.setdefault("monitor", {})
     for i in range(screen.get_n_monitors()):
-        m_info[i] = get_monitor_info(display, screen, i)
+        m_info[i] = get_screen_monitor_info(screen, i)
     fo = screen.get_font_options()
     #win32 and osx return nothing here...
     if fo:
@@ -466,15 +478,15 @@ def get_visual_info(v):
             vinfo[x] = vdict.get(val, val)
     return vinfo
 
-def get_monitor_info(_display, screen, i) -> dict:
+def get_screen_monitor_info(screen, i) -> dict:
     info = {}
     geom = screen.get_monitor_geometry(i)
     for x in ("x", "y", "width", "height"):
         info[x] = getattr(geom, x)
     if hasattr(screen, "get_monitor_plug_name"):
         info["plug_name"] = screen.get_monitor_plug_name(i) or ""
-    for x in ("scale_factor", "width_mm", "height_mm"):
-        fn = getattr(screen, "get_monitor_"+x, None)
+    for x in ("scale_factor", "width_mm", "height_mm", "refresh_rate"):
+        fn = getattr(screen, "get_monitor_"+x, None) or getattr(screen, "get_"+x, None)
         if fn:
             info[x] = int(fn(i))
     rectangle = screen.get_monitor_workarea(i)
@@ -483,17 +495,60 @@ def get_monitor_info(_display, screen, i) -> dict:
         workarea_info[x] = getattr(rectangle, x)
     return info
 
-
-def get_display_info() -> dict:
+def get_monitors_info(xscale=1, yscale=1):
     display = Gdk.Display.get_default()
+    info = {}
+    n = display.get_n_monitors()
+    for i in range(n):
+        minfo = info.setdefault(i, {})
+        monitor = display.get_monitor(i)
+        minfo["primary"] = monitor.is_primary()
+        for attr in (
+            "geometry", "refresh-rate", "scale-factor",
+            "width-mm", "height-mm",
+            "manufacturer", "model",
+            "subpixel-layout",  "workarea",
+            ):
+            getter = getattr(monitor, "get_%s" % attr.replace("-", "_"), None)
+            if getter:
+                value = getter()
+                if value is None:
+                    continue
+                if isinstance(value, Gdk.Rectangle):
+                    value = (round(xscale*value.x), round(yscale*value.y), round(xscale*value.width), round(yscale*value.height))
+                elif attr=="width-mm":
+                    value = round(xscale*value)
+                elif attr=="height-mm":
+                    value = round(yscale*value)
+                elif attr=="subpixel-layout":
+                    value = {
+                        Gdk.SubpixelLayout.UNKNOWN          : "unknown",
+                        Gdk.SubpixelLayout.NONE             : "none",
+                        Gdk.SubpixelLayout.HORIZONTAL_RGB   : "horizontal-rgb",
+                        Gdk.SubpixelLayout.HORIZONTAL_BGR   : "horizontal-bgr",
+                        Gdk.SubpixelLayout.VERTICAL_RGB     : "vertical-rgb",
+                        Gdk.SubpixelLayout.VERTICAL_BGR     : "vertical-bgr",
+                        }.get(value, "unknown")
+                if isinstance(value, str):
+                    value = value.strip()
+                minfo[attr] = value
+    return info
+
+def get_display_info(xscale=1, yscale=1) -> dict:
+    display = Gdk.Display.get_default()
+    def xy(v):
+        return round(xscale*v[0]), round(yscale*v[1])
+    def avg(v):
+        return round((xscale*v+yscale*v)/2)
+    root_size = get_root_size()
     info = {
-            "root-size"             : get_root_size(),
+            "root-size"             : xy(root_size),
             "screens"               : display.get_n_screens(),
             "name"                  : display.get_name(),
-            "pointer"               : display.get_pointer()[-3:-1],
+            "pointer"               : xy(display.get_pointer()[-3:-1]),
             "devices"               : len(display.list_devices()),
-            "default_cursor_size"   : display.get_default_cursor_size(),
-            "maximal_cursor_size"   : display.get_maximal_cursor_size(),
+            "default_cursor_size"   : avg(display.get_default_cursor_size()),
+            "maximal_cursor_size"   : xy(display.get_maximal_cursor_size()),
             "pointer_is_grabbed"    : display.pointer_is_grabbed(),
             }
     if not WIN32:
@@ -505,10 +560,13 @@ def get_display_info() -> dict:
             fn = getattr(display, f)
             sinfo[x]  = fn()
     info["screens"] = get_screens_info()
+    info["monitors"] = get_monitors_info(xscale, yscale)
     dm = display.get_device_manager()
-    for dt, name in {Gdk.DeviceType.MASTER  : "master",
-                     Gdk.DeviceType.SLAVE   : "slave",
-                     Gdk.DeviceType.FLOATING: "floating"}.items():
+    for dt, name in {
+        Gdk.DeviceType.MASTER  : "master",
+        Gdk.DeviceType.SLAVE   : "slave",
+        Gdk.DeviceType.FLOATING: "floating",
+        }.items():
         dinfo = info.setdefault("device", {})
         dtinfo = dinfo.setdefault(name, {})
         devices = dm.list_devices(dt)
@@ -533,7 +591,7 @@ def get_icon_from_file(filename):
         if not os.path.exists(filename):
             log.warn("Warning: cannot load icon, '%s' does not exist", filename)
             return None
-        with open(filename, mode='rb') as f:
+        with open(filename, mode="rb") as f:
             data = f.read()
         loader = GdkPixbuf.PixbufLoader()
         loader.write(data)
@@ -709,13 +767,29 @@ def choose_file(parent_window, title, action=Gtk.FileChooserAction.OPEN, action_
     return filename
 
 
+def init_display_source():
+    """
+    On X11, we want to be able to access the bindings
+    so we need to get the X11 display from GDK.
+    """
+    if is_X11():
+        from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+        init_gdk_display_source()
+
+
 def main():
     from xpra.platform import program_context
     from xpra.log import enable_color
     with program_context("GTK-Version-Info", "GTK Version Info"):
         enable_color()
         print("%s" % get_gtk_version_info())
-        get_screen_sizes()
+        if POSIX and not OSX:
+            from xpra.x11.bindings.posix_display_source import init_posix_display_source    #@UnresolvedImport
+            init_posix_display_source()
+        import warnings
+        warnings.simplefilter("ignore")
+        print(get_screen_sizes()[0])
+        print_nested_dict(get_display_info())
 
 
 if __name__ == "__main__":
