@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,10 +8,11 @@ import os
 import subprocess
 import hashlib
 import uuid
+from time import monotonic
 
 from xpra.child_reaper import getChildReaper
-from xpra.os_util import monotonic_time, bytestostr, strtobytes, umask_context, POSIX, WIN32
-from xpra.util import typedict, csv, envint, envbool, engs
+from xpra.os_util import bytestostr, memoryview_to_bytes, umask_context, POSIX, WIN32
+from xpra.util import typedict, csv, envint, envbool, engs, net_utf8, u
 from xpra.scripts.config import parse_bool, parse_with_unit
 from xpra.simple_stats import std_unit
 from xpra.make_thread import start_thread
@@ -91,18 +92,6 @@ def safe_open_download_file(basefilename, mimetype):
     filelog("using filename '%s', file descriptor=%s", filename, fd)
     return filename, fd
 
-def s(v):
-    try:
-        return v.decode("utf8")
-    except (AttributeError, UnicodeDecodeError):
-        return bytestostr(v)
-
-def utf8_decode(url):
-    try:
-        return strtobytes(url).decode("utf8")
-    except UnicodeDecodeError:
-        return bytestostr(url)
-
 
 class FileTransferAttributes:
 
@@ -250,7 +239,7 @@ class FileTransferHandler(FileTransferAttributes):
                 self.remote_open_url, self.remote_open_url_ask)
 
     def get_info(self) -> dict:
-        info = FileTransferAttributes.get_info(self)
+        info = super().get_info()
         info["remote"] = {
             "file-transfer"     : self.remote_file_transfer,
             "file-transfer-ask" : self.remote_file_transfer_ask,
@@ -267,23 +256,30 @@ class FileTransferHandler(FileTransferAttributes):
         return info
 
 
-    def digest_mismatch(self, filename, digest, expected_digest, algo="sha1"):
-        filelog.error("Error: data does not match, invalid %s file digest for '%s'", algo, filename)
-        filelog.error(" received %s, expected %s", digest, expected_digest)
-        raise Exception("failed %s digest verification" % algo)
+    def digest_mismatch(self, filename, digest, expected_digest):
+        filelog.error("Error: data does not match, invalid %s file digest for '%s'", digest.name, filename)
+        filelog.error(f" received {digest.hexdigest()}")
+        filelog.error(f" expected {expected_digest}")
+        try:
+            if os.path.exists(filename):
+                os.unlink(filename)
+        except OSError:
+            filelog.error(f"Error: failed to delete uploaded file {filename}")
 
 
     def _check_chunk_receiving(self, chunk_id, chunk_no):
         chunk_state = self.receive_chunks_in_progress.get(chunk_id)
         filelog("_check_chunk_receiving(%s, %s) chunk_state=%s", chunk_id, chunk_no, chunk_state)
-        if chunk_state:
-            if chunk_state[-4]:
-                #transfer has been cancelled
-                return
-            chunk_state[-2] = 0     #this timer has been used
-            if chunk_state[-1]==0:
-                filelog.error("Error: chunked file transfer '%s' timed out", chunk_id)
-                self.receive_chunks_in_progress.pop(chunk_id, None)
+        if not chunk_state:
+            #transfer not found
+            return
+        if chunk_state[-4]:
+            #transfer has been cancelled
+            return
+        chunk_state[-2] = 0     #this timer has been used
+        if chunk_state[-1]==chunk_no:
+            filelog.error("Error: chunked file transfer '%s' timed out", chunk_id)
+            self.receive_chunks_in_progress.pop(chunk_id, None)
 
     def cancel_download(self, send_id, message="Cancelled"):
         filelog("cancel_download(%s, %s)", send_id, message)
@@ -291,7 +287,7 @@ class FileTransferHandler(FileTransferAttributes):
             if chunk_state[-3]==send_id:
                 self.cancel_file(chunk_id, message)
                 return
-        filelog.error("Error: cannot cancel download %s, entry not found!", s(send_id))
+        filelog.error("Error: cannot cancel download %s, entry not found!", u(send_id))
 
     def cancel_file(self, chunk_id, message, chunk=0):
         filelog("cancel_file%s", (chunk_id, message, chunk))
@@ -322,7 +318,7 @@ class FileTransferHandler(FileTransferAttributes):
 
     def _process_send_file_chunk(self, packet):
         chunk_id, chunk, file_data, has_more = packet[1:5]
-        chunk_id = bytestostr(chunk_id)
+        chunk_id = net_utf8(chunk_id)
         filelog("_process_send_file_chunk%s", (chunk_id, chunk, "%i bytes" % len(file_data), has_more))
         chunk_state = self.receive_chunks_in_progress.get(chunk_id)
         if not chunk_state:
@@ -336,7 +332,7 @@ class FileTransferHandler(FileTransferAttributes):
             start = chunk_state[0]
             send_id = chunk_state[-3]
             filesize = chunk_state[6]
-            self.transfer_progress_update(False, send_id, monotonic_time()-start, position, filesize, error)
+            self.transfer_progress_update(False, send_id, monotonic()-start, position, filesize, error)
         fd = chunk_state[1]
         if chunk_state[-1]+1!=chunk:
             filelog.error("Error: chunk number mismatch, expected %i but got %i", chunk_state[-1]+1, chunk)
@@ -344,13 +340,15 @@ class FileTransferHandler(FileTransferAttributes):
             osclose(fd)
             progress(-1, "chunk no mismatch")
             return
+        file_data = memoryview_to_bytes(file_data)
         #update chunk number:
         chunk_state[-1] = chunk
         digest = chunk_state[8]
         written = chunk_state[9]
         try:
             os.write(fd, file_data)
-            digest.update(file_data)
+            if digest:
+                digest.update(file_data)
             written += len(file_data)
             chunk_state[9] = written
         except OSError as e:
@@ -360,7 +358,16 @@ class FileTransferHandler(FileTransferAttributes):
             osclose(fd)
             progress(-1, "write error (%s)" % e)
             return
+        filesize = chunk_state[6]
+        if written>filesize:
+            filelog.error("Error: too much data received")
+            progress(-1, "file data size mismatch")
+            return
         self.send("ack-file-chunk", chunk_id, True, "", chunk)
+        if chunk_state[-4]:
+            #check again if the transfer has been cancelled
+            filelog("got chunk for a cancelled file transfer, ignoring it")
+            return
         if has_more:
             progress(written)
             timer = chunk_state[-2]
@@ -370,23 +377,27 @@ class FileTransferHandler(FileTransferAttributes):
             timer = self.timeout_add(CHUNK_TIMEOUT, self._check_chunk_receiving, chunk_id, chunk)
             chunk_state[-2] = timer
             return
+        #we have received all the packets
         self.receive_chunks_in_progress.pop(chunk_id, None)
         osclose(fd)
+        if digest:
+            options = chunk_state[7]
+            expected_digest = options.strget(digest.name)   #ie: "sha256"
+            filename = chunk_state[2]
+            if expected_digest and digest.hexdigest()!=expected_digest:
+                progress(-1, "checksum mismatch")
+                self.digest_mismatch(filename, digest, expected_digest)
+                return
+            filelog("%s digest matches: %s", digest.name, expected_digest)
         #check file size and digest then process it:
         filename, mimetype, printit, openit, filesize, options = chunk_state[2:8]
         if written!=filesize:
             filelog.error("Error: expected a file of %i bytes, got %i", filesize, written)
             progress(-1, "file size mismatch")
             return
-        expected_digest = options.strget("sha1")
-        if expected_digest and digest.hexdigest()!=expected_digest:
-            progress(-1, "checksum mismatch")
-            self.digest_mismatch(filename, digest, expected_digest, "sha1")
-            return
-
         progress(written)
         start_time = chunk_state[0]
-        elapsed = monotonic_time()-start_time
+        elapsed = monotonic()-start_time
         mimetype = bytestostr(mimetype)
         filelog("%i bytes received in %i chunks, took %ims", filesize, chunk, elapsed*1000)
         self.process_downloaded_file(filename, mimetype, printit, openit, filesize, options)
@@ -418,14 +429,14 @@ class FileTransferHandler(FileTransferAttributes):
 
     def _process_send_file(self, packet):
         #the remote end is sending us a file
-        start = monotonic_time()
+        start = monotonic()
         basefilename, mimetype, printit, openit, filesize, file_data, options = packet[1:8]
         send_id = ""
         if len(packet)>=9:
-            send_id = s(packet[8])
+            send_id = net_utf8(packet[8])
         #basefilename should be utf8:
-        basefilename = utf8_decode(basefilename)
-        mimetype = bytestostr(mimetype)
+        basefilename = net_utf8(basefilename)
+        mimetype = net_utf8(mimetype)
         if filesize<=0:
             filelog.error("Error: invalid file size: %s", filesize)
             filelog.error(" file transfer aborted for %r", basefilename)
@@ -436,7 +447,7 @@ class FileTransferHandler(FileTransferAttributes):
         if r is None:
             filelog.warn("Warning: %s rejected for file '%s'",
                          ("transfer", "printing")[bool(printit)],
-                         bytestostr(basefilename))
+                         basefilename)
             return
         #accept_data can override the flags:
         printit, openit = r
@@ -450,7 +461,7 @@ class FileTransferHandler(FileTransferAttributes):
         l("receiving file: %s",
           [basefilename, mimetype, printit, openit, filesize, "%s bytes" % len(file_data), options])
         if filesize>self.file_size_limit:
-            l.error("Error: file '%s' is too large:", s(basefilename))
+            l.error("Error: file '%s' is too large:", basefilename)
             l.error(" %sB, the file size limit is %sB",
                     std_unit(filesize), std_unit(self.file_size_limit))
             return
@@ -465,17 +476,21 @@ class FileTransferHandler(FileTransferAttributes):
                 self.send("ack-file-chunk", chunk_id, False, "failed to create file: %s" % e, 0)
             return
         self.file_descriptors.add(fd)
+        digest = None
+        for hash_fn in ("sha512", "sha384", "sha256", "sha224", "sha1"):
+            if options.get(hash_fn):
+                digest = getattr(hashlib, hash_fn)()
+                break
         if chunk_id:
+            chunk = 0
             l = len(self.receive_chunks_in_progress)
             if l>=MAX_CONCURRENT_FILES:
-                self.send("ack-file-chunk", chunk_id, False, "too many file transfers in progress: %i" % l, 0)
-                os.close(fd)
+                self.cancel_file(chunk_id, "too many file transfers in progress: %i" % l, chunk)
+                osclose(fd)
                 return
-            digest = hashlib.sha1()
-            chunk = 0
             timer = self.timeout_add(CHUNK_TIMEOUT, self._check_chunk_receiving, chunk_id, chunk)
             chunk_state = [
-                monotonic_time(),
+                monotonic(),
                 fd, filename, mimetype,
                 printit, openit, filesize,
                 options, digest, 0, False, send_id,
@@ -487,25 +502,22 @@ class FileTransferHandler(FileTransferAttributes):
         #not chunked, full file:
         assert file_data, "no data, got %s" % (file_data,)
         if len(file_data)!=filesize:
-            l.error("Error: invalid data size for file '%s'", s(basefilename))
+            l.error("Error: invalid data size for file '%s'", basefilename)
             l.error(" received %i bytes, expected %i bytes", len(file_data), filesize)
             return
         #check digest if present:
-        def check_digest(algo="sha1", libfn=hashlib.sha1):
-            digest = options.get(algo)
-            if digest:
-                u = libfn()
-                u.update(file_data)
-                l("%s digest: %s - expected: %s", algo, u.hexdigest(), digest)
-                if digest!=u.hexdigest():
-                    self.digest_mismatch(filename, digest, u.hexdigest(), algo)
-        check_digest("sha1", hashlib.sha1)
-        check_digest("md5", hashlib.md5)
+        if digest:
+            digest.update(file_data)
+            expected_digest = options.strget(digest.name)   #ie: "sha256"
+            if expected_digest and digest.hexdigest()!=expected_digest:
+                self.digest_mismatch(basefilename, digest, expected_digest)
+                return
+            filelog("%s digest matches: %s", digest.name, expected_digest)
         try:
             os.write(fd, file_data)
         finally:
             os.close(fd)
-        self.transfer_progress_update(False, send_id, monotonic_time()-start, filesize, filesize, None)
+        self.transfer_progress_update(False, send_id, monotonic()-start, filesize, filesize, None)
         self.process_downloaded_file(filename, mimetype, printit, openit, filesize, options)
 
 
@@ -582,14 +594,14 @@ class FileTransferHandler(FileTransferAttributes):
             printlog("printing failed and returned %i", job)
             delfile()
             return
-        start = monotonic_time()
+        start = monotonic()
         def check_printing_finished():
             done = printing_finished(job)
             printlog("printing_finished(%s)=%s", job, done)
             if done:
                 delfile()
                 return False
-            if monotonic_time()-start>=PRINT_JOB_TIMEOUT:
+            if monotonic()-start>=PRINT_JOB_TIMEOUT:
                 printlog.warn("Warning: print job %s timed out", job)
                 delfile()
                 return False
@@ -617,20 +629,20 @@ class FileTransferHandler(FileTransferAttributes):
             #which may point back to us!
             self.exec_open_command(url)
         else:
-            import webbrowser
+            import webbrowser  #pylint: disable=import-outside-toplevel
             webbrowser.open_new_tab(url)
 
     def exec_open_command(self, url):
         filelog("exec_open_command(%s)", url)
         try:
-            import shlex
+            import shlex  #pylint: disable=import-outside-toplevel
             command = shlex.split(self.open_command)+[url]
         except ImportError as e:
             filelog("exec_open_command(%s) no shlex: %s", url, e)
             command = self.open_command.split(" ")
         filelog("exec_open_command(%s) command=%s", url, command)
         try:
-            proc = subprocess.Popen(command, env=self.get_open_env(), shell=WIN32)
+            proc = subprocess.Popen(command, env=self.get_open_env(), shell=WIN32)  # pylint: disable=consider-using-with
         except Exception as e:
             filelog("exec_open_command(%s)", url, exc_info=True)
             filelog.error("Error: cannot open '%s': %s", url, e)
@@ -667,8 +679,8 @@ class FileTransferHandler(FileTransferAttributes):
 
 
     def _process_open_url(self, packet):
-        send_id = s(packet[2])
-        url = utf8_decode(packet[1])
+        send_id = net_utf8(packet[2])
+        url = net_utf8(packet[1])
         if not self.open_url:
             filelog.warn("Warning: received a request to open URL '%s'", url)
             filelog.warn(" but opening of URLs is disabled")
@@ -685,14 +697,15 @@ class FileTransferHandler(FileTransferAttributes):
             return False
         if self.remote_open_url_ask:
             #ask the client if it is OK to send
-            return self.send_data_request("open", b"url", url)
+            return self.send_data_request("open", "url", url)
         self.do_send_open_url(url)
         return True
 
     def do_send_open_url(self, url, send_id=""):
         self.send("open-url", url, send_id)
 
-    def send_file(self, filename, mimetype, data, filesize=0, printit=False, openit=False, options=None):
+    def send_file(self, filename, mimetype, data, filesize=0,
+                  printit=False, openit=False, options=None):
         if printit:
             l = printlog
             if not self.printing:
@@ -716,7 +729,7 @@ class FileTransferHandler(FileTransferAttributes):
             if openit:
                 if not self.remote_open_files:
                     l.info("opening the file after transfer is disabled on the remote end")
-                    l.info(" sending only, the file will need to be opended manually")
+                    l.info(" sending only, the file will need to be opened manually")
                     openit = False
                     action = "upload"
                 else:
@@ -726,19 +739,17 @@ class FileTransferHandler(FileTransferAttributes):
         data = data[:filesize]          #gio may null terminate it
         l("send_file%s action=%s, ask=%s",
           (filename, mimetype, type(data), "%i bytes" % filesize, printit, openit, options), action, ask)
-        try:
-            filename = bytestostr(filename).encode("utf8")
-        except Exception:
-            filename = strtobytes(filename)
         self.dump_remote_caps()
         if not self.check_file_size(action, filename, filesize):
             return False
         if ask:
-            return self.send_data_request(action, b"file", filename, mimetype, data, filesize, printit, openit, options)
-        self.do_send_file(filename, mimetype, data, filesize, printit, openit, options)
+            return self.send_data_request(action, "file", filename, mimetype, data, filesize, printit, openit, options)
+        send_id = uuid.uuid4().hex
+        self.do_send_file(filename, mimetype, data, filesize, printit, openit, options, send_id)
         return True
 
-    def send_data_request(self, action, dtype, url, mimetype="", data="", filesize=0, printit=False, openit=True, options=None):
+    def send_data_request(self, action, dtype, url, mimetype="", data="", filesize=0,
+                          printit=False, openit=True, options=None):
         send_id = uuid.uuid4().hex
         if len(self.pending_send_data)>=MAX_CONCURRENT_FILES:
             filelog.warn("Warning: %s dropped", action)
@@ -749,7 +760,7 @@ class FileTransferHandler(FileTransferAttributes):
         delay = self.remote_file_ask_timeout*1000
         self.pending_send_data_timers[send_id] = self.timeout_add(delay, self.send_data_ask_timeout, send_id)
         filelog("sending data request for %s '%s' with send-id=%s",
-                s(dtype), s(url), send_id)
+                u(dtype), u(url), send_id)
         self.send("send-data-request", dtype, send_id, url, mimetype, filesize, printit, openit, options or {})
         return send_id
 
@@ -760,15 +771,15 @@ class FileTransferHandler(FileTransferAttributes):
         if len(packet)>=9:
             options = packet[8]
         #filenames and url are always sent encoded as utf8:
-        url = utf8_decode(url)
-        dtype = s(dtype)
-        send_id = s(send_id)
+        url = net_utf8(url)
+        dtype = net_utf8(dtype)
+        send_id = net_utf8(send_id)
         self.do_process_send_data_request(dtype, send_id, url, _, filesize, printit, openit, typedict(options))
 
 
     def do_process_send_data_request(self, dtype, send_id, url, _, filesize, printit, openit, options):
         filelog("do_process_send_data_request: send_id=%s, url=%s, printit=%s, openit=%s, options=%s",
-                s(send_id), url, printit, openit, options)
+                u(send_id), url, printit, openit, options)
         def cb_answer(accept):
             filelog("accept%s=%s", (url, printit, openit), accept)
             self.send("send-data-response", send_id, accept)
@@ -819,8 +830,8 @@ class FileTransferHandler(FileTransferAttributes):
 
     def _process_send_data_response(self, packet):
         send_id, accept = packet[1:3]
-        filelog("process send-data-response: send_id=%s, accept=%s", s(send_id), accept)
-        send_id = s(send_id)
+        send_id = net_utf8(send_id)
+        filelog("process send-data-response: send_id=%s, accept=%s", send_id, accept)
         timer = self.pending_send_data_timers.pop(send_id, None)
         if timer:
             self.source_remove(timer)
@@ -828,13 +839,13 @@ class FileTransferHandler(FileTransferAttributes):
         if v is None:
             filelog.warn("Warning: cannot find send-file entry")
             return
-        dtype = v[0]
-        url = v[1]
+        dtype = net_utf8(v[0])
+        url = net_utf8(v[1])
         if accept==DENY:
-            filelog.info("the request to send %s '%s' has been denied", bytestostr(dtype), s(url))
+            filelog.info("the request to send %s '%s' has been denied", dtype, url)
             return
         assert accept in (ACCEPT, OPEN), "unknown value for send-data response: %s" % (accept,)
-        if dtype==b"file":
+        if dtype=="file":
             mimetype, data, filesize, printit, openit, options = v[2:]
             if accept==ACCEPT:
                 self.do_send_file(url, mimetype, data, filesize, printit, openit, options, send_id)
@@ -842,7 +853,7 @@ class FileTransferHandler(FileTransferAttributes):
                 assert openit and accept==OPEN
                 #try to open at this end:
                 self._open_file(url)
-        elif dtype==b"url":
+        elif dtype=="url":
             if accept==ACCEPT:
                 self.do_send_open_url(url, send_id)
             else:
@@ -871,15 +882,15 @@ class FileTransferHandler(FileTransferAttributes):
         else:
             action = "upload"
             l = filelog
-        l("do_send_file%s", (s(filename), mimetype, type(data), "%i bytes" % filesize, printit, openit, options))
+        l("do_send_file%s", (u(filename), mimetype, type(data), "%i bytes" % filesize, printit, openit, options))
         if not self.check_file_size(action, filename, filesize):
             return False
-        u = hashlib.sha1()
-        u.update(data)
+        h = hashlib.sha256()
+        h.update(data)
         absfile = os.path.abspath(filename)
-        filelog("sha1 digest('%s')=%s", s(absfile), u.hexdigest())
+        filelog("sha256 digest('%s')=%s", u(absfile), h.hexdigest())
         options = options or {}
-        options["sha1"] = u.hexdigest()
+        options["sha256"] = h.hexdigest()
         chunk_size = min(self.file_chunks, self.remote_file_chunks)
         if 0<chunk_size<filesize:
             if len(self.send_chunks_in_progress)>=MAX_CONCURRENT_FILES:
@@ -888,8 +899,9 @@ class FileTransferHandler(FileTransferAttributes):
             chunk_id = uuid.uuid4().hex
             options["file-chunk-id"] = chunk_id
             #timer to check that the other end is requesting more chunks:
-            timer = self.timeout_add(CHUNK_TIMEOUT, self._check_chunk_sending, chunk_id, 0)
-            chunk_state = [monotonic_time(), data, chunk_size, timer, 0]
+            chunk_no = 0
+            timer = self.timeout_add(CHUNK_TIMEOUT, self._check_chunk_sending, chunk_id, chunk_no)
+            chunk_state = [monotonic(), data, chunk_size, timer, chunk_no]
             self.send_chunks_in_progress[chunk_id] = chunk_state
             cdata = ""
             filelog("using chunks, sending initial file-chunk-id=%s, for chunk size=%s",
@@ -900,42 +912,40 @@ class FileTransferHandler(FileTransferAttributes):
             assert len(cdata)<=filesize     #compressed wrapper ensures this is true
             filelog("sending full file: %i bytes (chunk size=%i)", filesize, chunk_size)
         basefilename = os.path.basename(filename)
-        #convert str to utf8 bytes:
-        try:
-            base = basefilename.encode("utf8")
-        except (AttributeError, UnicodeEncodeError):
-            base = strtobytes(basefilename)
-        self.send("send-file", base, mimetype, printit, openit, filesize, cdata, options, send_id)
+        self.send("send-file", basefilename, mimetype, printit, openit, filesize, cdata, options, send_id)
         return True
 
     def _check_chunk_sending(self, chunk_id, chunk_no):
         chunk_state = self.send_chunks_in_progress.get(chunk_id)
         filelog("_check_chunk_sending(%s, %s) chunk_state found: %s", chunk_id, chunk_no, bool(chunk_state))
-        if chunk_state:
-            chunk_state[3] = 0         #timer has fired
-            if chunk_state[-1]==chunk_no:
-                filelog.error("Error: chunked file transfer '%s' timed out", chunk_id)
-                filelog.error(" on chunk %i", chunk_no)
-                self.cancel_sending(chunk_id)
+        if not chunk_state:
+            #transfer already removed
+            return
+        chunk_state[3] = 0         #timer has fired
+        if chunk_state[-1]==chunk_no:
+            filelog.error("Error: chunked file transfer '%s' timed out", chunk_id)
+            filelog.error(" on chunk %i", chunk_no)
+            self.cancel_sending(chunk_id)
 
     def cancel_sending(self, chunk_id):
         chunk_state = self.send_chunks_in_progress.pop(chunk_id, None)
         filelog("cancel_sending(%s) chunk state found: %s", chunk_id, bool(chunk_state))
-        if chunk_state:
-            timer = chunk_state[3]
-            if timer:
-                chunk_state[3] = 0
-                self.source_remove(timer)
+        if not chunk_state:
+            return
+        timer = chunk_state[3]
+        if timer:
+            chunk_state[3] = 0
+            self.source_remove(timer)
 
     def _process_ack_file_chunk(self, packet):
         #the other end received our send-file or send-file-chunk,
         #send some more file data
         filelog("ack-file-chunk: %s", packet[1:])
         chunk_id, state, error_message, chunk = packet[1:5]
-        chunk_id = bytestostr(chunk_id)
+        chunk_id = net_utf8(chunk_id)
         if not state:
             filelog.info("the remote end is cancelling the file transfer:")
-            filelog.info(" %s", bytestostr(error_message))
+            filelog.info(" %s", net_utf8(error_message))
             self.cancel_sending(chunk_id)
             return
         chunk_state = self.send_chunks_in_progress.get(chunk_id)
@@ -943,13 +953,13 @@ class FileTransferHandler(FileTransferAttributes):
             filelog.error("Error: cannot find the file transfer id '%r'", chunk_id)
             return
         if chunk_state[-1]!=chunk:
-            filelog.error("Error: chunk number mismatch (%i vs %i)", chunk_state, chunk)
+            filelog.error("Error: chunk number mismatch (%i vs %i)", chunk_state[-1], chunk)
             self.cancel_sending(chunk_id)
             return
         start_time, data, chunk_size, timer, chunk = chunk_state
         if not data:
             #all sent!
-            elapsed = monotonic_time()-start_time
+            elapsed = monotonic()-start_time
             filelog("%i chunks of %i bytes sent in %ims (%sB/s)",
                     chunk, chunk_size, elapsed*1000, std_unit(chunk*chunk_size/elapsed))
             self.cancel_sending(chunk_id)
