@@ -1,7 +1,7 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # This file is part of Xpra.
-# Copyright (C) 2009-2016 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2009-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -18,8 +18,8 @@ log = Logger("network", "mdns")
 
 class AvahiListener:
 
-    def __init__(self, service_type, mdns_found=None, mdns_add=None, mdns_remove=None):
-        log("AvahiListener%s", (service_type, mdns_found, mdns_add, mdns_remove))
+    def __init__(self, service_type, mdns_found=None, mdns_add=None, mdns_remove=None, mdns_update=None):
+        log("AvahiListener%s", (service_type, mdns_found, mdns_add, mdns_remove, mdns_update))
         try:
             self.bus = init_system_bus()
             assert self.bus
@@ -35,6 +35,7 @@ class AvahiListener:
         self.mdns_found = mdns_found
         self.mdns_add = mdns_add
         self.mdns_remove = mdns_remove
+        #self.mdns_update = mdns_update
         self.server = None
 
     def resolve_error(self, *args):
@@ -42,7 +43,7 @@ class AvahiListener:
 
     def service_resolved(self, interface, protocol, name, stype, domain, host, x, address, port, text_array, v):
         log("AvahiListener.service_resolved%s",
-		(interface, protocol, name, stype, domain, host, x, address, port, "..", v))
+        (interface, protocol, name, stype, domain, host, x, address, port, "..", v))
         if self.mdns_add:
             #parse text data:
             text = {}
@@ -114,10 +115,9 @@ def main():
     loop_init()
     listener = AvahiListener(XPRA_MDNS_TYPE, mdns_found, mdns_add, mdns_remove)
     try:
-        from xpra.gtk_common.gobject_compat import import_glib
-        glib = import_glib()
-        glib.idle_add(listener.start)
-        glib.MainLoop().run()
+        from gi.repository import GLib
+        GLib.idle_add(listener.start)
+        GLib.MainLoop().run()
     finally:
         listener.stop()
 
