@@ -116,12 +116,6 @@ def do_get_download_dir():
     return d
 
 
-def get_libexec_dir():
-    return env_or_delegate("XPRA_LIBEXEC_DIR", do_get_libexec_dir)
-def do_get_libexec_dir():
-    return get_app_dir()
-
-
 def get_mmap_dir():
     return env_or_delegate("XPRA_MMAP_DIR", do_get_mmap_dir)
 def do_get_mmap_dir():
@@ -282,14 +276,24 @@ def get_sound_command():
 def do_get_sound_command():
     return get_xpra_command()
 
+
+def get_python_exec_command():
+    envvalue = os.environ.get("XPRA_PYTHON_EXEC_COMMAND") or os.environ.get("XPRA_PYTHON_COMMAND")
+    if envvalue:
+        import shlex
+        return shlex.split(envvalue)
+    return do_get_python_exec_command()
+def do_get_python_exec_command():
+    return ["python%i.%i" % (sys.version_info.major, sys.version_info.minor), "-c"]
+
 def get_python_execfile_command():
-    envvalue = os.environ.get("XPRA_PYTHON_COMMAND")
+    envvalue = os.environ.get("XPRA_PYTHON_EXECFILE_COMMAND") or os.environ.get("XPRA_PYTHON_COMMAND")
     if envvalue:
         import shlex
         return shlex.split(envvalue)
     return do_get_python_execfile_command()
 def do_get_python_execfile_command():
-    return ["python3"]
+    return ["python%i.%i" % (sys.version_info.major, sys.version_info.minor)]
 
 
 platform_import(globals(), "paths", True,
@@ -314,11 +318,11 @@ platform_import(globals(), "paths", False,
                 "do_get_client_socket_dirs",
                 "do_get_default_log_dirs",
                 "do_get_download_dir",
-                "do_get_libexec_dir",
                 "do_get_mmap_dir",
                 "do_get_xpra_tmp_dir",
                 "do_get_script_bin_dirs",
                 "do_get_desktop_background_paths",
+                "do_get_python_exec_command",
                 "do_get_python_execfile_command",
                 )
 
@@ -344,7 +348,6 @@ def get_info():
         "client-socket"     : {"dirs"   : get_client_socket_dirs()},
         "log"               : {"dirs"   : get_default_log_dirs()},
         "download"          : {"dir"    : get_download_dir()},
-        "libexec"           : {"dir"    : get_libexec_dir()},
         "mmap"              : {"dir"    : get_mmap_dir()},
         "xpra-tmp"          : {"dir"    : get_xpra_tmp_dir()},
         "script"            : {"dir"    : get_script_bin_dirs()},
@@ -359,6 +362,7 @@ def get_info():
         "nodock_command"    : get_nodock_command(),
         "sound_command"     : get_sound_command(),
         "sshpass_command"   : get_sshpass_command(),
+        "python-exec"       : get_python_exec_command(),
         "python-execfile"   : get_python_execfile_command(),
         }
 
