@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -7,7 +7,7 @@ import os
 import sys
 
 from xpra.util import envbool
-from xpra.os_util import strtobytes
+from xpra.os_util import strtobytes, getuid
 from xpra.scripts.config import parse_bool
 from xpra.server.auth.sys_auth_base import SysAuthenticator, log
 
@@ -17,7 +17,7 @@ PAM_CHECK_ACCOUNT = envbool("XPRA_PAM_CHECK_ACCOUNT", False)
 
 def check(username, password, service=PAM_AUTH_SERVICE, check_account=PAM_CHECK_ACCOUNT):
     log("pam check(%s, [..])", username)
-    from xpra.server.pam import pam_session #@UnresolvedImport
+    from xpra.server.pam import pam_session #@UnresolvedImport pylint: disable=import-outside-toplevel
     b = strtobytes
     session = pam_session(b(username), b(password), service)
     if not session.start(b(password)):
@@ -35,11 +35,12 @@ def check(username, password, service=PAM_AUTH_SERVICE, check_account=PAM_CHECK_
 
 
 class Authenticator(SysAuthenticator):
+    CLIENT_USERNAME = getuid()==0
 
-    def __init__(self, username, **kwargs):
+    def __init__(self, **kwargs):
         self.service = kwargs.pop("service", PAM_AUTH_SERVICE)
         self.check_account = parse_bool("check-account", kwargs.pop("check-account", PAM_CHECK_ACCOUNT), False)
-        super().__init__(username, **kwargs)
+        super().__init__(**kwargs)
 
     def check(self, password) -> bool:
         log("pam.check(..) pw=%s", self.pw)
@@ -63,13 +64,13 @@ def main(args):
         print("usage:")
         print("%s username password" % (args[0],))
         return 1
-    a = Authenticator(args[1])
+    username = args[1]
+    a = Authenticator(username=username)
     if a.check(args[2]):
         print("success")
         return 0
-    else:
-        print("failed")
-        return -1
+    print("failed")
+    return -1
 
 if __name__ == "__main__":
     sys.exit(main(sys.argv))
