@@ -1,14 +1,16 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 from collections import deque
 
+from xpra.platform.info import get_username
 from xpra.platform.dotxpra import DotXpra
 from xpra.platform.paths import get_socket_dirs
-from xpra.util import envint, obsc, typedict
+from xpra.util import envint, obsc, typedict, std
+from xpra.scripts.config import TRUE_OPTIONS
 from xpra.net.digest import get_salt, choose_digest, verify_digest, gendigest
 from xpra.os_util import hexstr, POSIX
 from xpra.log import Logger
@@ -30,7 +32,7 @@ def parse_uid(v) -> int:
             log("uid '%s' is not an int", v)
     if POSIX:
         try:
-            import pwd
+            import pwd  #pylint: disable=import-outside-toplevel
             return pwd.getpwnam(v or DEFAULT_UID).pw_uid
         except Exception as e:
             log("parse_uid(%s)", v, exc_info=True)
@@ -46,7 +48,7 @@ def parse_gid(v) -> int:
             log("gid '%s' is not an int", v)
     if POSIX:
         try:
-            import grp          #@UnresolvedImport
+            import grp          #@UnresolvedImport pylint: disable=import-outside-toplevel
             return grp.getgrnam(v or DEFAULT_GID).gr_gid
         except Exception as e:
             log("parse_gid(%s)", v, exc_info=True)
@@ -57,22 +59,30 @@ def parse_gid(v) -> int:
 
 class SysAuthenticatorBase:
     USED_SALT = deque(maxlen=USED_SALT_CACHE_SIZE)
+    DEFAULT_PROMPT = "password for user '{username}'"
+    CLIENT_USERNAME = False
 
-    def __init__(self, username, **kwargs):
-        self.username = username
+    def __init__(self, **kwargs):
+        self.username = kwargs.get("username", get_username())
+        if str(kwargs.get("client-username", self.CLIENT_USERNAME)).lower() in TRUE_OPTIONS:
+            #allow the client to specify the username to authenticate with:
+            self.username = kwargs.get("remote", {}).get("username", self.username)
         self.salt = None
         self.digest = None
         self.salt_digest = None
-        self.prompt = kwargs.pop("prompt", "password")
+        prompt_attr = {"username" : std(self.username)}
+        self.prompt = kwargs.pop("prompt", self.DEFAULT_PROMPT).format(**prompt_attr)
         self.socket_dirs = kwargs.pop("socket-dirs", get_socket_dirs())
         self.challenge_sent = False
         self.passed = False
         self.password_used = None
-        #warn about unused options:
-        unused = dict((k,v) for k,v in kwargs.items() if k not in ("connection", "exec_cwd"))
-        if unused:
-            log.warn("Warning: unused keyword arguments for %s authentication:", self)
-            log.warn(" %s", unused)
+        #we can't warn about unused options
+        #because the options are shared with other socket options (nodelay, cork, etc)
+        #unused = dict((k,v) for k,v in kwargs.items() if k not in ("connection", "exec_cwd", "username"))
+        #if unused:
+        #    log.warn("Warning: unused keyword arguments for %s authentication:", self)
+        #    log.warn(" %s", unused)
+        log("auth prompt=%s, socket_dirs=%s", self.prompt, self.socket_dirs)
 
     def get_uid(self) -> int:
         raise NotImplementedError()
@@ -213,15 +223,15 @@ class SysAuthenticatorBase:
 
 class SysAuthenticator(SysAuthenticatorBase):
 
-    def __init__(self, username, **kwargs):
-        super().__init__(username, **kwargs)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
         self.pw = None
         if POSIX:
             try:
-                import pwd
-                self.pw = pwd.getpwnam(username)
+                import pwd  #pylint: disable=import-outside-toplevel
+                self.pw = pwd.getpwnam(self.username)
             except Exception:
-                log("cannot load password database entry for '%s'", username, exc_info=True)
+                log("cannot load password database entry for '%s'", self.username, exc_info=True)
 
     def get_uid(self) -> int:
         if self.pw is None:
