@@ -1,15 +1,18 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2015-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2015-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import dbus.service  # @UnresolvedImport
+from dbus import PROPERTIES_IFACE  # @UnresolvedImport
+from dbus.exceptions import DBusException  # @UnresolvedImport
+
 from xpra.dbus.helper import dbus_to_native
 from xpra.dbus.common import init_session_bus
 from xpra.util import AtomicInteger, DETACH_REQUEST
-import dbus.service
-
 from xpra.log import Logger
+
 log = Logger("dbus", "server")
 
 BUS_NAME = "org.xpra.Server"
@@ -39,7 +42,7 @@ class DBUS_Source(dbus.service.Object):
         if extra:
             name += extra.replace(".", "_").replace(":", "_")
         bus_name = dbus.service.BusName(name, session_bus)
-        dbus.service.Object.__init__(self, bus_name, self.path)
+        super().__init__(bus_name, self.path)
         self.log("(%s)", source)
         self._properties = {"bell"              : ("send_bell",             ni),
                             "cursors"           : ("send_cursors",          ni),
@@ -63,17 +66,17 @@ class DBUS_Source(dbus.service.Object):
         log("%s"+fmt, INTERFACE, *args)
 
 
-    @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature='s', out_signature='v')
+    @dbus.service.method(PROPERTIES_IFACE, in_signature='s', out_signature='v')
     def Get(self, property_name):
         conv = self._properties.get(property_name)
         if conv is None:
-            raise dbus.exceptions.DBusException("invalid property")
+            raise DBusException("invalid property")
         server_property_name, _ = conv
-        v = getattr(self.server, server_property_name)
+        v = getattr(self.source, server_property_name)
         self.log(".Get(%s)=%s", property_name, v)
         return v
 
-    @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature='', out_signature='a{sv}')
+    @dbus.service.method(PROPERTIES_IFACE, in_signature='', out_signature='a{sv}')
     def GetAll(self, interface_name):
         if interface_name==INTERFACE:
             v = dict((x, self.Get(x)) for x in self._properties.keys())
@@ -82,17 +85,17 @@ class DBUS_Source(dbus.service.Object):
         self.log(".GetAll(%s)=%s", interface_name, v)
         return v
 
-    @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature='ssv')
+    @dbus.service.method(PROPERTIES_IFACE, in_signature='ssv')
     def Set(self, interface_name, property_name, new_value):
         self.log(".Set(%s, %s, %s)", interface_name, property_name, new_value)
         conv = self._properties.get(property_name)
         if conv is None:
-            raise dbus.exceptions.DBusException("invalid property")
+            raise DBusException("invalid property")
         server_property_name, validator = conv
-        assert hasattr(self.server, server_property_name)
-        setattr(self.server, server_property_name, validator(new_value))
+        assert hasattr(self.source, server_property_name)
+        setattr(self.source, server_property_name, validator(new_value))
 
-    @dbus.service.signal(dbus.PROPERTIES_IFACE, signature='sa{sv}as')
+    @dbus.service.signal(PROPERTIES_IFACE, signature='sa{sv}as')
     def PropertiesChanged(self, interface_name, changed_properties, invalidated_properties):
         pass
 
