@@ -11,7 +11,7 @@ from gi.repository import GLib
 
 from xpra.common import DEFAULT_XDG_DATA_DIRS
 from xpra.os_util import (
-    OSX, POSIX,
+    OSX, POSIX, WIN32,
     osexpand,
     )
 from xpra.util import envint, envbool
@@ -21,6 +21,7 @@ from xpra.log import Logger
 
 log = Logger("menu")
 
+MENU_WATCHER = envbool("XPRA_MENU_WATCHER", True)
 MENU_RELOAD_DELAY = envint("XPRA_MENU_RELOAD_DELAY", 5)
 EXPORT_XDG_MENU_DATA = envbool("XPRA_EXPORT_XDG_MENU_DATA", True)
 
@@ -63,7 +64,8 @@ class MenuProvider:
     def setup(self):
         if OSX or not EXPORT_XDG_MENU_DATA:
             return
-        self.setup_menu_watcher()
+        if MENU_WATCHER:
+            self.setup_menu_watcher()
         self.load_menu_data()
 
     def cleanup(self):
@@ -81,10 +83,11 @@ class MenuProvider:
             log.error(" %s", e)
 
     def do_setup_menu_watcher(self):
-        if self.watch_manager:
+        if self.watch_manager or OSX or WIN32:
             #already setup
             return
         try:
+            # pylint: disable=import-outside-toplevel
             import pyinotify
         except ImportError as e:
             log("setup_menu_watcher() cannot import pyinotify", exc_info=True)
@@ -103,7 +106,7 @@ class MenuProvider:
         mask = pyinotify.IN_DELETE | pyinotify.IN_CREATE  #@UndefinedVariable pylint: disable=no-member
         handler = EventHandler()
         self.watch_notifier = pyinotify.ThreadedNotifier(self.watch_manager, handler)
-        self.watch_notifier.setDaemon(True)
+        self.watch_notifier.daemon = True
         data_dirs = os.environ.get("XDG_DATA_DIRS", DEFAULT_XDG_DATA_DIRS).split(":")
         watched = []
         for data_dir in data_dirs:
@@ -138,9 +141,16 @@ class MenuProvider:
         #as this may take a while and
         #so server startup can complete:
         def load():
-            self.get_menu_data(force_reload)
-            self.get_desktop_sessions()
-            self.clear_cache()
+            try:
+                self.get_menu_data(force_reload)
+                self.get_desktop_sessions()
+            except ImportError as e:
+                log.warn("Warning: cannot load menu data")
+                log.warn(f" {e}")
+            except Exception:
+                log.error("Error loading menu data", exc_info=True)
+            finally:
+                self.clear_cache()
         start_thread(load, "load-menu-data", True)
 
     def get_menu_data(self, force_reload=False, remove_icons=False, wait=True):
