@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 #pylint: disable-msg=E1101
@@ -8,10 +8,11 @@
 import sys
 import logging
 import traceback
+from time import monotonic
 from threading import Lock
 
-from xpra.os_util import bytestostr, strtobytes, monotonic_time
-from xpra.util import repr_ellipsized
+from xpra.os_util import bytestostr
+from xpra.util import repr_ellipsized, net_utf8
 from xpra.scripts.config import FALSE_OPTIONS, TRUE_OPTIONS
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
 from xpra.log import Logger, set_global_logging_handler
@@ -19,10 +20,10 @@ from xpra.log import Logger, set_global_logging_handler
 log = Logger("server")
 
 
-"""
-Mixin for servers that can receive and send logging packets
-"""
 class LoggingServer(StubServerMixin):
+    """
+    Mixin for servers that can receive and send logging packets
+    """
 
     def __init__(self):
         self.remote_logging_send = False
@@ -69,7 +70,7 @@ class LoggingServer(StubServerMixin):
             log.warn("Warning: logging already enabled for client %s", protocol)
         else:
             log.info("sending log output to %s", protocol)
-            self.logging_clients[protocol] = monotonic_time()
+            self.logging_clients[protocol] = monotonic()
         if n==0:
             self.start_capturing_logging()
 
@@ -90,18 +91,14 @@ class LoggingServer(StubServerMixin):
         if self.in_remote_logging:
             return
         assert self.local_logging
-        def enc(x):
-            try:
-                return bytestostr(x).encode("utf8")
-            except UnicodeEncodeError:
-                return strtobytes(x)
         def local_warn(*args):
             self.local_logging(log, logging.WARNING, *args)
-        def local_err(message):
+        def local_err(message, e=None):
             if self._closing:
                 return
             local_warn("Warning: %s:", message)
-            local_warn(" %s" % e)
+            if e:
+                local_warn(" %s", e)
             local_warn(" original unformatted message: %s", msg)
             if args:
                 local_warn(" %i arguments: %s", len(args), args)
@@ -122,31 +119,35 @@ class LoggingServer(StubServerMixin):
         try:
             try:
                 if args:
-                    s = msg % args
+                    data = msg % args
                 else:
-                    s = msg
+                    data = msg
             except Exception as e:
-                local_err("failed to format log message")
+                local_err("failed to format log message", e)
                 return
             for proto, start_time in self.logging_clients.items():
                 source = self.get_server_source(proto)
                 if not source:
                     continue
                 try:
-                    dtime = int(1000*(monotonic_time() - start_time))
-                    data = source.compressed_wrapper("text", enc(s))
+                    dtime = int(1000*(monotonic() - start_time))
+                    if len(data)>=32:
+                        try:
+                            data = source.compressed_wrapper("text", data.encode("utf8"), level=1)
+                        except Exception:
+                            pass
                     source.send("logging", level, data, dtime)
                     exc_info = kwargs.get("exc_info")
                     if exc_info is True:
                         exc_info = sys.exc_info()
                     if exc_info and exc_info[0]:
                         for x in traceback.format_tb(exc_info[2]):
-                            self.send("logging", level, enc(x), dtime)
+                            self.send("logging", level, x, dtime)
                         try:
                             etypeinfo = exc_info[0].__name__
                         except AttributeError:
                             etypeinfo = str(exc_info[0])
-                        source.send("logging", level, enc("%s: %s" % (etypeinfo, exc_info[1])), dtime)
+                        source.send("logging", level, "%s: %s" % (etypeinfo, exc_info[1]), dtime)
                 except Exception as e:
                     if self._closing:
                         return
@@ -182,16 +183,11 @@ class LoggingServer(StubServerMixin):
         if len(packet)>=4:
             dtime = packet[3]
             prefix += "@%02i.%03i " % ((dtime//1000)%60, dtime%1000)
-        def dec(x):
-            try:
-                return x.decode("utf8")
-            except Exception:
-                return bytestostr(x)
         try:
             if isinstance(msg, (tuple, list)):
-                dmsg = " ".join(dec(x) for x in msg)
+                dmsg = " ".join(net_utf8(x) for x in msg)
             else:
-                dmsg = dec(msg)
+                dmsg = net_utf8(msg)
             for l in dmsg.splitlines():
                 self.do_log(level, prefix+l)
         except Exception as e:
