@@ -1,17 +1,18 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 #pylint: disable-msg=E1101
 
 import os.path
+from time import monotonic
 
-from xpra.util import parse_scaling_value, from0to100
-from xpra.server.control_command import ArgsControlCommand, ControlError
+from xpra.util import parse_scaling_value, csv, from0to100, net_utf8, typedict
 from xpra.os_util import load_binary_file
-from xpra.util import csv
+from xpra.simple_stats import std_unit
 from xpra.scripts.config import parse_bool, FALSE_OPTIONS, TRUE_OPTIONS
+from xpra.server.control_command import ArgsControlCommand, ControlError
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
 from xpra.log import Logger
 
@@ -23,10 +24,10 @@ TOGGLE_FEATURES = (
     )
 
 
-"""
-Control commands for ServerBase
-"""
 class ServerBaseControlCommands(StubServerMixin):
+    """
+    Control commands for ServerBase
+    """
 
     def setup(self):
         self.add_control_commands()
@@ -58,13 +59,17 @@ class ServerBaseControlCommands(StubServerMixin):
 
         for cmd in (
             ArgsControlCommand("focus",                 "give focus to the window id",      validation=[int]),
+            ArgsControlCommand("map",                   "maps the window id",               validation=[int]),
+            ArgsControlCommand("unmap",                 "unmaps the window id",             validation=[int]),
             #window source:
             ArgsControlCommand("suspend",               "suspend screen updates",           max_args=0),
             ArgsControlCommand("resume",                "resume screen updates",            max_args=0),
             ArgsControlCommand("ungrab",                "cancels any grabs",                max_args=0),
             #server globals:
+            ArgsControlCommand("readonly",              "set readonly state for client(s)", min_args=1, max_args=1, validation=[parse_boolean_value]),
             ArgsControlCommand("idle-timeout",          "set the idle tiemout",             validation=[int]),
             ArgsControlCommand("server-idle-timeout",   "set the server idle timeout",      validation=[int]),
+            ArgsControlCommand("start-env",             "modify the environment used to start new commands", min_args=2),
             ArgsControlCommand("start",                 "executes the command arguments in the server context", min_args=1),
             ArgsControlCommand("start-child",           "executes the command arguments in the server context, as a 'child' (honouring exit-with-children)", min_args=1),
             ArgsControlCommand("toggle-feature",        "toggle a server feature on or off, one of: %s" % csv(TOGGLE_FEATURES), min_args=1, max_args=2, validation=[str, parse_boolean_value]),
@@ -89,11 +94,17 @@ class ServerBaseControlCommands(StubServerMixin):
             ArgsControlCommand("sound-output",          "control sound forwarding",         min_args=1, max_args=2),
             #windows:
             ArgsControlCommand("workspace",             "move a window to a different workspace", min_args=2, max_args=2, validation=[int, int]),
+            ArgsControlCommand("close",                 "close a window",                   min_args=1, max_args=1, validation=[int]),
+            ArgsControlCommand("delete",                "delete a window",                   min_args=1, max_args=1, validation=[int]),
+            ArgsControlCommand("move",                  "move a window",                    min_args=3, max_args=3, validation=[int, int, int]),
+            ArgsControlCommand("resize",                "resize a window",                  min_args=3, max_args=3, validation=[int, int, int]),
+            ArgsControlCommand("moveresize",            "move and resize a window",         min_args=5, max_args=5, validation=[int, int, int, int, int]),
             ArgsControlCommand("scaling-control",       "set the scaling-control aggressiveness (from 0 to 100)", min_args=1, validation=[from0to100]),
             ArgsControlCommand("scaling",               "set a specific scaling value",     min_args=1, validation=[parse_scaling_value]),
             ArgsControlCommand("auto-refresh",          "set a specific auto-refresh value", min_args=1, validation=[float]),
             ArgsControlCommand("refresh",               "refresh some or all windows",      min_args=0),
-            ArgsControlCommand("encoding",              "picture encoding",                 min_args=1, max_args=1),
+            ArgsControlCommand("encoding",              "picture encoding",                 min_args=2),
+            ArgsControlCommand("request-update",        "request a screen update using a specific encoding",  min_args=3),
             ArgsControlCommand("video-region-enabled",  "enable video region",              min_args=2, max_args=2, validation=[int, parse_boolean_value]),
             ArgsControlCommand("video-region-detection","enable video detection",           min_args=2, max_args=2, validation=[int, parse_boolean_value]),
             ArgsControlCommand("video-region-exclusion-zones","set window regions to exclude from video regions: 'WID,(x,y,w,h),(x,y,w,h),..', ie: '1 (0,10,100,20),(200,300,20,20)'",  min_args=2, max_args=2, validation=[int, parse_4intlist]),
@@ -107,7 +118,10 @@ class ServerBaseControlCommands(StubServerMixin):
             cmd.do_run = getattr(self, "control_command_%s" % cmd.name.replace("-", "_"))
             self.control_commands[cmd.name] = cmd
         #encoding bits:
-        for name in ("quality", "min-quality", "speed", "min-speed"):
+        for name in (
+            "quality", "min-quality", "max-quality",
+            "speed", "min-speed", "max-speed",
+            ):
             fn = getattr(self, "control_command_%s" % name.replace("-", "_"))
             self.control_commands[name] = ArgsControlCommand(name, "set encoding %s (from 0 to 100)" % name, run=fn, min_args=1, validation=[from0to100])
 
@@ -122,6 +136,40 @@ class ServerBaseControlCommands(StubServerMixin):
         self._focus(None, wid, None)
         return "gave focus to window %s" % wid
 
+    def control_command_map(self, wid):
+        if self.readonly:
+            return
+        assert type(wid)==int, "argument should have been an int, but found %s" % type(wid)
+        window = self._id_to_window.get(wid)
+        assert window, "window %i not found" % wid
+        if window.is_tray():
+            return "cannot map tray window %s" % wid
+        if window.is_OR():
+            return "cannot map override redirect window %s" % wid
+        window.show()
+        #window.set_owner(dm)
+        #iconic = window.get_property("iconic")
+        #if iconic:
+        #    window.set_property("iconic", False)
+        #w, h = window.get_geometry()[2:4]
+        #self.refresh_window_area(window, 0, 0, w, h)
+        self.repaint_root_overlay()
+        return "mapped window %s" % wid
+
+    def control_command_unmap(self, wid):
+        if self.readonly:
+            return
+        assert type(wid)==int, "argument should have been an int, but found %s" % type(wid)
+        window = self._id_to_window.get(wid)
+        assert window, "window %i not found" % wid
+        if window.is_tray():
+            return "cannot map tray window %s" % wid
+        if window.is_OR():
+            return "cannot map override redirect window %s" % wid
+        window.hide()
+        self.repaint_root_overlay()
+        return "unmapped window %s" % wid
+
     def control_command_suspend(self):
         for csource in tuple(self._server_sources.values()):
             csource.suspend(True, self._id_to_window)
@@ -137,6 +185,13 @@ class ServerBaseControlCommands(StubServerMixin):
             csource.pointer_ungrab(-1)
         return "ungrabbed %s clients" % len(self._server_sources)
 
+    def control_command_readonly(self, onoff):
+        log("control_command_readonly(%s)", onoff)
+        self.readonly = onoff
+        msg = "server readonly: %s" % onoff
+        log.info(msg)
+        return msg
+
     def control_command_idle_timeout(self, t):
         self.idle_timeout = t
         for csource in tuple(self._server_sources.values()):
@@ -150,6 +205,21 @@ class ServerBaseControlCommands(StubServerMixin):
         self.reset_server_timeout(reschedule)
         return "server-idle-timeout set to %s" % t
 
+
+    def control_command_start_env(self, action="set", var_name="", value=None):
+        assert var_name, "the environment variable name must be specified"
+        if action=="unset":
+            assert value is None, "invalid number of arguments for %s" % action
+            if self.start_env.pop(var_name, None) is None:
+                return "%r is not set" % var_name
+            return "%r unset" % var_name
+        if action=="set":
+            assert value, "the value must be specified"
+            self.start_env[var_name] = value
+            return "%s=%s" % (var_name, value)
+        return "invalid start-env subcommand %r" % action
+
+
     def control_command_start(self, *args):
         return self.do_control_command_start(True, *args)
     def control_command_start_child(self, *args):
@@ -236,11 +306,16 @@ class ServerBaseControlCommands(StubServerMixin):
             raise ControlError("no clients found matching: %s" % client_uuids)
         clients = 0
         for ss in sources:
-            if ss.send_open_url(url):
-                clients += 1
+            if hasattr(ss, "send_open_url"):
+                if ss.send_open_url(url):
+                    clients += 1
         return "url sent to %i clients" % clients
 
     def control_command_send_file(self, filename, openit="open", client_uuids="*", maxbitrate=0):
+        #we always get the values as strings from the command interface,
+        #but those may actually be utf8 encoded binary strings,
+        #so we may have to do an ugly roundtrip:
+        filename = net_utf8(filename)
         openit = str(openit).lower() in ("open", "true", "1")
         return self.do_control_file_command("send file", client_uuids, filename, "file_transfer", (False, openit))
 
@@ -260,10 +335,9 @@ class ServerBaseControlCommands(StubServerMixin):
         if not sources:
             raise ControlError("no clients found matching: %s" % client_uuids)
         def checksize(file_size):
-            file_size_MB = file_size//1024//1024
-            if file_size_MB>self.file_transfer.file_size_limit:
-                raise ControlError("file '%s' is too large: %iMB (limit is %iMB)" % (
-                    filename, file_size_MB, self.file_transfer.file_size_limit))
+            if file_size>self.file_transfer.file_size_limit:
+                raise ControlError("file '%s' is too large: %sB (limit is %sB)" % (
+                    filename, std_unit(file_size), std_unit(self.file_transfer.file_size_limit)))
 
         #find the file and load it:
         actual_filename = os.path.abspath(os.path.expanduser(filename))
@@ -281,17 +355,23 @@ class ServerBaseControlCommands(StubServerMixin):
             raise ControlError("failed to load '%s'" % actual_filename)
         #verify size:
         file_size = len(data)
-        file_size_MB = file_size//1024//1024
         checksize(file_size)
         #send it to each client:
         for ss in sources:
             #ie: ServerSource.file_transfer (found in FileTransferAttributes)
-            if not getattr(ss, source_flag_name):
+            if not getattr(ss, source_flag_name, False):
+                #skip the warning if the client is not interactive
+                #(for now just check for 'top' client):
+                if ss.client_type=="top":
+                    l = log
+                else:
+                    l = log.warn
+                l("Warning: cannot %s '%s' to %s client", command_type, filename, ss.client_type)
+                l(" client %s does not support this feature", ss.uuid)
+            elif file_size>ss.file_size_limit:
                 log.warn("Warning: cannot %s '%s'", command_type, filename)
-                log.warn(" client %s does not support this feature", ss)
-            elif file_size_MB>ss.file_size_limit:
-                log.warn("Warning: cannot %s '%s'", command_type, filename)
-                log.warn(" client %s file size limit is %iMB (file is %iMB)", ss, ss.file_size_limit, file_size_MB)
+                log.warn(" client %s file size limit is %sB (file is %sB)",
+                         ss, std_unit(ss.file_size_limit), std_unit(file_size))
             else:
                 ss.send_file(filename, "", data, file_size, *send_file_args)
         return "%s of '%s' to %s initiated" % (command_type, filename, client_uuids)
@@ -305,7 +385,7 @@ class ServerBaseControlCommands(StubServerMixin):
         return "removed %i window-filters" % l
 
     def control_command_add_window_filter(self, object_name, property_name, operator, value, client_uuids=""):
-        from xpra.server.window import filters
+        from xpra.server.window import filters  #pylint: disable=import-outside-toplevel
         window_filter = filters.get_window_filter(object_name, property_name, operator, value)
         #log("%s%s=%s", filters.get_window_filter, (object_name, property_name, operator, value), window_filter)
         if client_uuids=="*":
@@ -319,8 +399,8 @@ class ServerBaseControlCommands(StubServerMixin):
 
     def control_command_compression(self, compression):
         c = compression.lower()
-        from xpra.net import compression
-        opts = compression.get_enabled_compressors()    #ie: [lz4, lzo, zlib]
+        from xpra.net import compression    #pylint: disable=import-outside-toplevel
+        opts = compression.get_enabled_compressors()    #ie: [lz4, zlib]
         if c not in opts:
             raise ControlError("compressor argument must be one of: %s" % csv(opts))
         for cproto in tuple(self._server_sources.keys()):
@@ -330,8 +410,8 @@ class ServerBaseControlCommands(StubServerMixin):
 
     def control_command_encoder(self, encoder):
         e = encoder.lower()
-        from xpra.net import packet_encoding
-        opts = packet_encoding.get_enabled_encoders()   #ie: [rencode, bencode, yaml]
+        from xpra.net import packet_encoding  #pylint: disable=import-outside-toplevel
+        opts = packet_encoding.get_enabled_encoders()   #ie: [rencode, rencodeplus, bencode, yaml]
         if e not in opts:
             raise ControlError("encoder argument must be one of: %s" % csv(opts))
         for cproto in tuple(self._server_sources.keys()):
@@ -374,7 +454,7 @@ class ServerBaseControlCommands(StubServerMixin):
         self.mdns_update()
         return "session name set to %s" % name
 
-    def _control_windowsources_from_args(self, *args):
+    def _ws_from_args(self, *args):
         #converts the args to valid window ids,
         #then returns all the window sources for those wids
         if len(args)==0 or len(args)==1 and args[0]=="*":
@@ -385,23 +465,24 @@ class ServerBaseControlCommands(StubServerMixin):
             for x in args:
                 try:
                     wid = int(x)
-                except:
-                    raise ControlError("invalid window id: %s" % x)
+                except ValueError:
+                    raise ControlError("invalid window id: %s" % x) from None
                 if wid in self._id_to_window:
                     wids.append(wid)
                 else:
                     log("window id %s does not exist", wid)
-        wss = {}
+        wss = []
         for csource in tuple(self._server_sources.values()):
             for wid in wids:
                 ws = csource.window_sources.get(wid)
                 window = self._id_to_window.get(wid)
                 if window and ws:
-                    wss[ws] = window
+                    wss.append(ws)
         return wss
 
+
     def _set_encoding_property(self, name, value, *wids):
-        for ws in self._control_windowsources_from_args(*wids).keys():
+        for ws in self._ws_from_args(*wids):
             fn = getattr(ws, "set_%s" % name.replace("-", "_"))   #ie: "set_quality"
             fn(value)
         #now also update the defaults:
@@ -413,46 +494,96 @@ class ServerBaseControlCommands(StubServerMixin):
         return self._set_encoding_property("quality", quality, *wids)
     def control_command_min_quality(self, min_quality, *wids):
         return self._set_encoding_property("min-quality", min_quality, *wids)
+    def control_command_max_quality(self, max_quality, *wids):
+        return self._set_encoding_property("max-quality", max_quality, *wids)
     def control_command_speed(self, speed, *wids):
         return self._set_encoding_property("speed", speed, *wids)
     def control_command_min_speed(self, min_speed, *wids):
         return self._set_encoding_property("min-speed", min_speed, *wids)
+    def control_command_max_speed(self, max_speed, *wids):
+        return self._set_encoding_property("max-speed", max_speed, *wids)
 
     def control_command_auto_refresh(self, auto_refresh, *wids):
         delay = int(float(auto_refresh)*1000.0)      # ie: 0.5 -> 500 (milliseconds)
-        for ws in self._control_windowsources_from_args(*wids).keys():
+        for ws in self._ws_from_args(*wids):
             ws.set_auto_refresh_delay(auto_refresh)
         return "auto-refresh delay set to %sms for windows %s" % (delay, wids)
 
     def control_command_refresh(self, *wids):
-        for ws in self._control_windowsources_from_args(*wids).keys():
+        for ws in self._ws_from_args(*wids):
             ws.full_quality_refresh({})
         return "refreshed windows %s" % str(wids)
 
     def control_command_scaling_control(self, scaling_control, *wids):
-        for ws in tuple(self._control_windowsources_from_args(*wids).keys()):
+        for ws in tuple(self._ws_from_args(*wids)):
             ws.set_scaling_control(scaling_control)
             ws.refresh()
         return "scaling-control set to %s on windows %s" % (scaling_control, wids)
 
     def control_command_scaling(self, scaling, *wids):
-        for ws in tuple(self._control_windowsources_from_args(*wids).keys()):
+        for ws in tuple(self._ws_from_args(*wids)):
             ws.set_scaling(scaling)
             ws.refresh()
         return "scaling set to %s on windows %s" % (str(scaling), wids)
 
     def control_command_encoding(self, encoding, *args):
+        if encoding in ("add", "remove"):
+            cmd = encoding
+            assert len(args)>0
+            encoding = args[0]
+            wids = args[1:]
+            for ws in tuple(self._ws_from_args(*wids)):
+                encodings = list(ws.encodings)
+                core_encodings = list(ws.core_encodings)
+                for l in (encodings, core_encodings):
+                    if cmd=="add" and encoding not in l:
+                        log("adding %s to %s for %s", encoding, l, ws)
+                        l.append(encoding)
+                    elif cmd=="remove" and encoding in l:
+                        log("removing %s from %s for %s", encoding, l, ws)
+                        l.remove(encoding)
+                    else:
+                        continue
+                ws.encodings = tuple(encodings)
+                ws.core_encodings = tuple(core_encodings)
+                ws.do_set_client_properties(typedict())
+                ws.refresh()
+            return "%s %s" % (["removed", "added"][cmd=="add"], encoding)
+
         strict = None       #means no change
-        if len(args)>0 and args[0] in ("strict", "nostrict"):
+        if encoding in ("strict", "nostrict"):
+            strict = encoding=="strict"
+            encoding = args[0]
+            wids = args[1:]
+        elif len(args)>0 and args[0] in ("strict", "nostrict"):
             #remove "strict" marker
             strict = args[0]=="strict"
-            args = args[1:]
-        wids = args
-        for ws in tuple(self._control_windowsources_from_args(*wids).keys()):
+            wids = args[1:]
+        else:
+            wids = args
+        for ws in tuple(self._ws_from_args(*wids)):
             ws.set_new_encoding(encoding, strict)
             ws.refresh()
         return "set encoding to %s%s for windows %s" % (encoding, ["", " (strict)"][int(strict or 0)], wids)
 
+    def control_command_request_update(self, encoding, geom, *args):
+        wids = args
+        now = monotonic()
+        options = {
+            "auto_refresh" : True,
+            "av-delay" : 0,
+            }
+        log("request-update using %r, geometry=%s, windows(%s)=%s",
+                 encoding, geom, wids, self._ws_from_args(*wids))
+        for ws in tuple(self._ws_from_args(*wids)):
+            if geom=="all":
+                x = y = 0
+                w, h = ws.window_dimensions
+            else:
+                x, y, w, h = (int(x) for x in geom.split(","))
+            ws.process_damage_region(now, x, y, w, h, encoding, options)
+        return "damage requested"
+
     def control_command_clipboard_direction(self, direction, *_args):
         ch = self._clipboard_helper
         assert self.clipboard and ch
@@ -481,7 +612,7 @@ class ServerBaseControlCommands(StubServerMixin):
         if wid not in self._id_to_window:
             raise ControlError("invalid window %i" % wid)
         video_subregions = []
-        for ws in self._control_windowsources_from_args(wid).keys():
+        for ws in self._ws_from_args(wid):
             vs = getattr(ws, "video_subregion", None)
             if not vs:
                 log.warn("Warning: cannot set video region enabled flag on window %i:", wid)
@@ -518,11 +649,11 @@ class ServerBaseControlCommands(StubServerMixin):
 
 
     def control_command_lock_batch_delay(self, wid, delay):
-        for ws in self._control_windowsources_from_args(wid).keys():
+        for ws in self._ws_from_args(wid):
             ws.lock_batch_delay(delay)
 
     def control_command_unlock_batch_delay(self, wid):
-        for ws in self._control_windowsources_from_args(wid).keys():
+        for ws in self._ws_from_args(wid):
             ws.unlock_batch_delay()
 
     def control_command_set_lock(self, lock):
@@ -555,9 +686,10 @@ class ServerBaseControlCommands(StubServerMixin):
                 keycode = int(keycode_str, 16)
             else:
                 keycode = int(keycode_str)
-            assert keycode>0 and keycode<=255
-        except:
-            raise ControlError("invalid keycode specified: '%s' (must be a number between 1 and 255)" % keycode_str)
+        except ValueError:
+            raise ControlError("invalid keycode specified: '%s' (not a number)" % keycode_str) from None
+        if keycode<=0 or keycode>=255:
+            raise ControlError("invalid keycode value: '%s' (must be between 1 and 255)" % keycode_str)
         if press is not True:
             if press in ("1", "press"):
                 press = True
@@ -586,6 +718,58 @@ class ServerBaseControlCommands(StubServerMixin):
         return "window %s moved to workspace %s" % (wid, workspace)
 
 
+    def control_command_close(self, wid):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        window.request_close()
+        return "requested window %s closed" % window
+
+    def control_command_delete(self, wid):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        window.send_delete()
+        return "requested window %s deleted" % window
+
+    def control_command_move(self, wid, x, y):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        ww, wh = window.get_dimensions()
+        count = 0
+        for source in tuple(self._server_sources.values()):
+            move_resize_window = getattr(source, "move_resize_window", None)
+            if move_resize_window:
+                move_resize_window(wid, window, x, y, ww, wh)
+                count += 1
+        return "window %s moved to %i,%i for %i clients" % (wid, x, y, count)
+
+    def control_command_resize(self, wid, w, h):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        count = 0
+        for source in tuple(self._server_sources.values()):
+            resize_window = getattr(source, "resize_window", None)
+            if resize_window:
+                resize_window(wid, window, w, h)
+                count += 1
+        return "window %s resized to %ix%i for %i clients" % (wid, w, h, count)
+
+    def control_command_moveresize(self, wid, x, y, w, h):
+        window = self._id_to_window.get(wid)
+        if not window:
+            raise ControlError("window %s does not exist" % wid)
+        count = 0
+        for source in tuple(self._server_sources.values()):
+            move_resize_window = getattr(source, "move_resize_window", None)
+            if move_resize_window:
+                move_resize_window(wid, window, x, y, w, h)
+                count += 1
+        return "window %s moved to %i,%i and resized to %ix%i for %i clients" % (wid, x, y, w, h, count)
+
+
     def _process_command_request(self, _proto, packet):
         """ client sent a command request through its normal channel """
         assert len(packet)>=2, "invalid command request packet (too small!)"
