@@ -1,30 +1,31 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import socket
-from time import sleep, time
+from time import sleep, time, monotonic
 from queue import Queue
 
 from xpra.net.net_util import get_network_caps
-from xpra.net.compression import Compressed, compressed_wrapper
-from xpra.net.protocol import Protocol
+from xpra.net.compression import Compressed, compressed_wrapper, MIN_COMPRESS_SIZE
+from xpra.net.protocol import CONNECTION_LOST
 from xpra.net.common import MAX_PACKET_SIZE
+from xpra.net.digest import get_salt, gendigest
 from xpra.codecs.loader import load_codec, get_codec
 from xpra.codecs.image_wrapper import ImageWrapper
 from xpra.codecs.video_helper import getVideoHelper, PREFERRED_ENCODER_ORDER
 from xpra.scripts.config import parse_number, parse_bool
+from xpra.common import FULL_INFO
 from xpra.os_util import (
-    get_hex_uuid,
-    monotonic_time, bytestostr, strtobytes,
+    get_hex_uuid, bytestostr, strtobytes,
     )
 from xpra.util import (
     flatten_dict, typedict, updict, ellipsizer, envint, envbool,
     csv, first_time, SERVER_SHUTDOWN,
     )
-from xpra.version_util import XPRA_VERSION
+from xpra.version_util import XPRA_VERSION, vparts
 from xpra.make_thread import start_thread
 from xpra.server.server_core import get_server_info, get_thread_info
 from xpra.log import Logger
@@ -52,22 +53,25 @@ class ProxyInstance:
 
     def __init__(self, session_options,
                  video_encoder_modules, pings,
-                 disp_desc, cipher, encryption_key, caps):
+                 disp_desc, cipher, cipher_mode, encryption_key, caps):
         self.session_options = session_options
         self.video_encoder_modules = video_encoder_modules
         self.pings = pings
         self.disp_desc = disp_desc
         self.cipher = cipher
+        self.cipher_mode = cipher_mode
         self.encryption_key = encryption_key
         self.caps = caps
         log("ProxyInstance%s", (
             session_options,
             video_encoder_modules,
-            disp_desc, cipher, encryption_key,
+            disp_desc, cipher, cipher_mode, encryption_key,
             "%s: %s.." % (type(caps), ellipsizer(caps))))
         self.uuid = get_hex_uuid()
         self.client_protocol = None
+        self.client_has_more = False
         self.server_protocol = None
+        self.server_has_more = False
         #ping handling:
         self.client_last_ping = 0
         self.server_last_ping = 0
@@ -104,12 +108,10 @@ class ProxyInstance:
         self.server_packets = Queue(PROXY_QUEUE_SIZE)
         self.client_packets = Queue(PROXY_QUEUE_SIZE)
         #server connection tweaks:
-        for x in (b"input-devices", b"draw", b"window-icon", b"keymap-changed", b"server-settings"):
-            self.server_protocol.large_packets.append(x)
+        self.server_protocol.large_packets += ["input-devices", "draw", "window-icon", "keymap-changed", "server-settings"]
         if self.caps.boolget("file-transfer"):
-            for x in (b"send-file", b"send-file-chunk"):
-                self.server_protocol.large_packets.append(x)
-                self.client_protocol.large_packets.append(x)
+            self.server_protocol.large_packets += ["send-file", "send-file-chunk"]
+            self.client_protocol.large_packets += ["send-file", "send-file-chunk"]
         self.server_protocol.set_compression_level(int(self.session_options.get("compression_level", 0)))
         self.server_protocol.enable_default_encoder()
 
@@ -197,7 +199,7 @@ class ProxyInstance:
             linfo["server"] = int(self.server_last_ping_latency)
         return {
             "proxy" : {
-                "version"    : XPRA_VERSION,
+                "version"    : vparts(XPRA_VERSION, FULL_INFO+1),
                 ""           : sinfo,
                 "latency"    : linfo,
                 },
@@ -206,7 +208,7 @@ class ProxyInstance:
 
     def get_window_info(self) -> dict:
         info = {}
-        now = monotonic_time()
+        now = monotonic()
         for wid, encoder in self.video_encoders.items():
             einfo = encoder.get_info()
             einfo["idle_time"] = int(now-self.video_encoders_last_used_time.get(wid, 0))
@@ -220,6 +222,16 @@ class ProxyInstance:
         return info
 
 
+    def get_connection_info(self) -> dict:
+        return {
+            "client" : self.client_protocol.get_info(),
+            "server" : self.server_protocol.get_info(),
+            }
+
+    def get_info(self) -> dict:
+        return {"connection" : self.get_connection_info()}
+
+
     ################################################################################
 
     def send_hello(self, challenge_response=None, client_salt=None):
@@ -239,9 +251,9 @@ class ProxyInstance:
         OPTION_WHITELIST = {
             "compression_level" : number,
             "lz4"               : parse_bool,
-            "lzo"               : parse_bool,
             "zlib"              : parse_bool,
             "rencode"           : parse_bool,
+            "rencodeplus"       : parse_bool,
             "bencode"           : parse_bool,
             "yaml"              : parse_bool,
             }
@@ -304,14 +316,15 @@ class ProxyInstance:
         p = self.client_packets.get()
         s = self.client_packets.qsize()
         log("sending to client: %s (queue size=%i)", bytestostr(p[0]), s)
-        return p, None, None, None, True, s>0
+        return p, None, None, None, True, s>0 or self.server_has_more
 
     def process_client_packet(self, proto, packet):
         packet_type = bytestostr(packet[0])
         log("process_client_packet: %s", packet_type)
-        if packet_type==Protocol.CONNECTION_LOST:
+        if packet_type==CONNECTION_LOST:
             self.stop(proto, "client connection lost")
             return
+        self.client_has_more = proto.receive_pending
         if packet_type=="set_deflate":
             #echo it back to the client:
             self.client_packets.put(packet)
@@ -333,7 +346,7 @@ class ProxyInstance:
         if packet_type=="ping_echo" and self.client_ping_timer and len(packet)>=7 and strtobytes(packet[6])==strtobytes(self.uuid):
             #this is one of our ping packets:
             self.client_last_ping_echo = packet[1]
-            self.client_last_ping_latency = 1000*monotonic_time()-self.client_last_ping_echo
+            self.client_last_ping_latency = 1000*monotonic()-self.client_last_ping_echo
             log("ping-echo: client latency=%.1fms", self.client_last_ping_latency)
             return
         #the packet types below are forwarded:
@@ -363,24 +376,17 @@ class ProxyInstance:
         p = self.server_packets.get()
         s = self.server_packets.qsize()
         log("sending to server: %s (queue size=%i)", bytestostr(p[0]), s)
-        return p, None, None, None, True, s>0
+        return p, None, None, None, True, s>0 or self.client_has_more
 
 
     def _packet_recompress(self, packet, index, name):
         if len(packet)>index:
             data = packet[index]
-            if len(data)<512:
-                packet[index] = strtobytes(data)
+            if len(data)<MIN_COMPRESS_SIZE:
                 return
             #this is ugly and not generic!
-            zlib = self.caps.boolget("zlib", True)
-            lz4 = self.caps.boolget("lz4", False)
-            lzo = self.caps.boolget("lzo", False)
-            if zlib or lz4 or lzo:
-                packet[index] = compressed_wrapper(name, data, zlib=zlib, lz4=lz4, lzo=lzo, can_inline=False)
-            else:
-                #prevent warnings about large uncompressed data
-                packet[index] = Compressed("raw %s" % name, data, can_inline=True)
+            kw = dict((k, self.caps.boolget(k)) for k in ("zlib", "lz4"))
+            packet[index] = compressed_wrapper(name, data, can_inline=False, **kw)
 
 
     def cancel_server_ping_timer(self):
@@ -400,13 +406,13 @@ class ProxyInstance:
     def schedule_server_ping(self):
         log("schedule_server_ping()")
         self.cancel_server_ping_timer()
-        self.server_last_ping_echo = monotonic_time()
+        self.server_last_ping_echo = monotonic()
         self.server_ping_timer = self.timeout_add(PING_INTERVAL, self.send_server_ping)
 
     def schedule_client_ping(self):
         log("schedule_client_ping()")
         self.cancel_client_ping_timer()
-        self.client_last_ping_echo = monotonic_time()
+        self.client_last_ping_echo = monotonic()
         self.client_ping_timer = self.timeout_add(PING_INTERVAL, self.send_client_ping)
 
     def send_server_ping(self):
@@ -420,7 +426,7 @@ class ProxyInstance:
                 log.error("Error: server ping timeout, %i seconds", delta)
                 self.stop(None, "proxy to server ping timeout")
                 return False
-        now = monotonic_time()
+        now = monotonic()
         self.server_last_ping = now
         self.queue_server_packet(("ping", int(now*1000), int(time()*1000), self.uuid))
         return True
@@ -436,7 +442,7 @@ class ProxyInstance:
                 log.error("Error: client ping timeout, %i seconds", delta)
                 self.stop(None, "proxy to client ping timeout")
                 return False
-        now = monotonic_time()
+        now = monotonic()
         self.client_last_ping = now
         self.queue_client_packet(("ping", int(now*1000), int(time()*1000), self.uuid))
         return True
@@ -445,9 +451,10 @@ class ProxyInstance:
     def process_server_packet(self, proto, packet):
         packet_type = bytestostr(packet[0])
         log("process_server_packet: %s", packet_type)
-        if packet_type==Protocol.CONNECTION_LOST:
+        if packet_type==CONNECTION_LOST:
             self.stop(proto, "server connection lost")
             return
+        self.server_has_more = proto.receive_pending
         if packet_type=="disconnect":
             reason = bytestostr(packet[1])
             log("got disconnect from server: %s", reason)
@@ -466,7 +473,8 @@ class ProxyInstance:
                 from xpra.net.crypto import crypto_backend_init, new_cipher_caps, DEFAULT_PADDING
                 crypto_backend_init()
                 padding_options = self.caps.strtupleget("cipher.padding.options", [DEFAULT_PADDING])
-                auth_caps = new_cipher_caps(self.client_protocol, self.cipher, self.encryption_key, padding_options)
+                auth_caps = new_cipher_caps(self.client_protocol,
+                                            self.cipher, self.cipher_mode, self.encryption_key, padding_options)
                 caps.update(auth_caps)
             #may need to bump packet size:
             proto.max_packet_size = max(MAX_PACKET_SIZE, maxw*maxh*4*4)
@@ -474,7 +482,7 @@ class ProxyInstance:
         elif packet_type=="ping_echo" and self.server_ping_timer and len(packet)>=7 and strtobytes(packet[6])==strtobytes(self.uuid):
             #this is one of our ping packets:
             self.server_last_ping_echo = packet[1]
-            self.server_last_ping_latency = 1000*monotonic_time()-self.server_last_ping_echo
+            self.server_last_ping_latency = 1000*monotonic()-self.server_last_ping_echo
             log("ping-echo: server latency=%.1fms", self.server_last_ping_latency)
             return
         elif packet_type=="info-response":
@@ -503,20 +511,14 @@ class ProxyInstance:
         #we do want to reformat cursor packets...
         #as they will have been uncompressed by the network layer already:
         elif packet_type=="cursor":
-            #packet = ["cursor", x, y, width, height, xhot, yhot, serial, pixels, name]
-            #or:
             #packet = ["cursor", "png", x, y, width, height, xhot, yhot, serial, pixels, name]
             #or:
             #packet = ["cursor", ""]
             if len(packet)>=8:
-                #hard to distinguish png cursors from normal cursors...
-                try:
-                    int(packet[1])
-                    self._packet_recompress(packet, 8, "cursor")
-                except (TypeError, ValueError):
-                    self._packet_recompress(packet, 9, "cursor")
+                self._packet_recompress(packet, 9, "cursor")
         elif packet_type=="window-icon":
-            self._packet_recompress(packet, 5, "icon")
+            if not isinstance(packet[5], str):
+                self._packet_recompress(packet, 5, "icon")
         elif packet_type=="send-file":
             if packet[6]:
                 packet[6] = Compressed("file-data", packet[6])
@@ -533,7 +535,6 @@ class ProxyInstance:
                 #otherwise, just forward it to the client
                 self.client_challenge_packet = packet
             else:
-                from xpra.net.digest import get_salt, gendigest
                 #client may have already responded to the challenge,
                 #so we have to handle authentication from this end
                 server_salt = bytestostr(packet[1])
@@ -579,14 +580,8 @@ class ProxyInstance:
     def encode_loop(self):
         """ thread for slower encoding related work """
         def delvideo(wid):
-            try:
-                del self.video_encoders[wid]
-            except KeyError:
-                pass
-            try:
-                del self.video_encoders_last_used_time[wid]
-            except KeyError:
-                pass
+            self.video_encoders.pop(wid, None)
+            self.video_encoders_last_used_time.pop(wid, None)
         while not self.exit:
             packet = self.encode_queue.get()
             if packet is None:
@@ -609,7 +604,7 @@ class ProxyInstance:
                     #not a real packet, this is added by the timeout check:
                     wid = packet[1]
                     ve = self.video_encoders.get(wid)
-                    now = monotonic_time()
+                    now = monotonic()
                     idle_time = now-self.video_encoders_last_used_time.get(wid, 0)
                     if ve and idle_time>VIDEO_TIMEOUT:
                         enclog("timing out the video encoder context for window %s", wid)
@@ -626,10 +621,14 @@ class ProxyInstance:
     def process_draw(self, packet):
         wid, x, y, width, height, encoding, pixels, _, rowstride, client_options = packet[1:11]
         encoding = bytestostr(encoding)
-        #never modify mmap packets
+        #never modify mmap or scroll packets:
         if encoding in ("mmap", "scroll"):
             return True
-
+        #we can only use video encoders on RGB data:
+        if encoding not in ("rgb24", "rgb32", "r210", "BGR565"):
+            #this prevents compression and inlining of pixel data:
+            packet[7] = Compressed("%s pixels" % encoding, pixels)
+            return True
         client_options = typedict(client_options)
         #we have a proxy video packet:
         rgb_format = client_options.strget("rgb_format", "")
@@ -652,8 +651,9 @@ class ProxyInstance:
                 if Xindex>=0 and len(rgb_format)==4:
                     #force clear alpha (which may be garbage):
                     newdata = bytearray(pixels)
+                    c = chr(255)
                     for i in range(len(pixels)/4):
-                        newdata[i*4+Xindex] = chr(255)
+                        newdata[i*4+Xindex] = c
                     packet[9] = client_options.intget("rowstride", 0)
                     cdata = bytes(newdata)
                 else:
@@ -665,10 +665,10 @@ class ProxyInstance:
                 new_client_options = client_options
             wrapped = Compressed("%s pixels" % encoding, cdata)
             #rgb32 is always supported by all clients:
-            return send_updated("rgb32", wrapped, new_client_options)
+            return send_updated(encoding, wrapped, new_client_options)
 
         proxy_video = client_options.boolget("proxy", False)
-        if PASSTHROUGH_RGB and (encoding in ("rgb32", "rgb24") or proxy_video):
+        if PASSTHROUGH_RGB:
             #we are dealing with rgb data, so we can pass it through:
             return passthrough(proxy_video)
         if not self.video_encoder_types or not client_options or not proxy_video:
@@ -739,7 +739,7 @@ class ProxyInstance:
                 dst_formats = self.video_encoders_dst_formats
             ve.init_context(width, height, rgb_format, dst_formats, encoding, quality, speed, scaling, {})
             self.video_encoders[wid] = ve
-            self.video_encoders_last_used_time[wid] = monotonic_time()      #just to make sure this is always set
+            self.video_encoders_last_used_time[wid] = monotonic()      #just to make sure this is always set
         #actual video compression:
         enclog("proxy compression using %s with quality=%s, speed=%s", ve, quality, speed)
         data, out_options = ve.compress_image(image, quality, speed, encoder_options)
@@ -748,14 +748,14 @@ class ProxyInstance:
         for k in ("timestamp", "rgb_format", "depth", "csc"):
             if k not in out_options and k in client_options:
                 out_options[k] = client_options[k]
-        self.video_encoders_last_used_time[wid] = monotonic_time()
+        self.video_encoders_last_used_time[wid] = monotonic()
         return send_updated(ve.get_encoding(), Compressed(encoding, data), out_options)
 
     def timeout_video_encoders(self):
         #have to be careful as another thread may come in...
         #so we just ask the encode thread (which deals with encoders already)
         #to do what may need to be done if we find a timeout:
-        now = monotonic_time()
+        now = monotonic()
         for wid in tuple(self.video_encoders_last_used_time.keys()):
             vetime = self.video_encoders_last_used_time.get(wid)
             if not vetime:
