@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from time import monotonic
 from collections import deque
 
 from xpra.server.source.stub_source_mixin import StubSourceMixin
 from xpra.platform.features import CLIPBOARDS
 from xpra.util import envint, typedict
-from xpra.os_util import monotonic_time
 from xpra.log import Logger
 
 log = Logger("clipboard")
@@ -25,9 +25,6 @@ class ClipboardConnection(StubSourceMixin):
     def is_needed(cls, caps : typedict) -> bool:
         return caps.boolget("clipboard")
 
-    def __init__(self, *_args):
-        pass
-
     def init_state(self):
         self.clipboard_enabled = False
         self.clipboard_notifications = False
@@ -37,28 +34,34 @@ class ClipboardConnection(StubSourceMixin):
         self.clipboard_stats = deque(maxlen=MAX_CLIPBOARD_LIMIT*MAX_CLIPBOARD_LIMIT_DURATION)
         self.clipboard_greedy = False
         self.clipboard_want_targets = False
-        self.clipboard_client_selections = CLIPBOARDS
+        self.clipboard_selections = CLIPBOARDS
         self.clipboard_preferred_targets = ()
-        self.clipboard_contents_slice_fix = False
 
     def cleanup(self):
         self.cancel_clipboard_progress_timer()
 
     def parse_client_caps(self, c : typedict):
-        self.clipboard_enabled = c.boolget("clipboard", False)
-        self.clipboard_notifications = c.boolget("clipboard.notifications")
+        ccaps = c.get("clipboard")
+        if ccaps and isinstance(ccaps, dict):
+            ccaps = typedict(ccaps)
+            self.clipboard_enabled = True
+            self.clipboard_notifications = ccaps.boolget("notifications")
+            self.clipboard_greedy = ccaps.boolget("greedy")
+            self.clipboard_want_targets = ccaps.boolget("want_targets")
+            self.clipboard_selections = ccaps.strtupleget("selections", CLIPBOARDS)
+            self.clipboard_preferred_targets = ccaps.strtupleget("preferred-targets", ())
+        else:
+            #no namespace in v4.3 and earlier:
+            self.clipboard_enabled = c.boolget("clipboard", False)
+            self.clipboard_notifications = c.boolget("clipboard.notifications")
+            self.clipboard_greedy = c.boolget("clipboard.greedy")
+            self.clipboard_want_targets = c.boolget("clipboard.want_targets")
+            self.clipboard_selections = c.strtupleget("clipboard.selections", CLIPBOARDS)
+            self.clipboard_preferred_targets = c.strtupleget("clipboard.preferred-targets", ())
         log("client clipboard: enabled=%s, notifications=%s",
             self.clipboard_enabled, self.clipboard_notifications)
-        self.clipboard_greedy = c.boolget("clipboard.greedy")
-        self.clipboard_want_targets = c.boolget("clipboard.want_targets")
-        self.clipboard_client_selections = c.strtupleget("clipboard.selections", CLIPBOARDS)
-        self.clipboard_contents_slice_fix = c.boolget("clipboard.contents-slice-fix")
-        self.clipboard_preferred_targets = c.strtupleget("clipboard.preferred-targets", ())
-        log("client clipboard: greedy=%s, want_targets=%s, client_selections=%s, contents_slice_fix=%s",
-            self.clipboard_greedy, self.clipboard_want_targets,
-            self.clipboard_client_selections, self.clipboard_contents_slice_fix)
-        if self.clipboard_enabled and not self.clipboard_contents_slice_fix:
-            log.info("client clipboard does not include contents slice fix")
+        log("client clipboard: greedy=%s, want_targets=%s, selections=%s",
+            self.clipboard_greedy, self.clipboard_want_targets, self.clipboard_selections)
 
     def get_info(self) -> dict:
         return {
@@ -68,8 +71,7 @@ class ClipboardConnection(StubSourceMixin):
                 "greedy"                : self.clipboard_greedy,
                 "want-targets"          : self.clipboard_want_targets,
                 "preferred-targets"     : self.clipboard_preferred_targets,
-                "selections"            : self.clipboard_client_selections,
-                "contents-slice-fix"    : self.clipboard_contents_slice_fix,
+                "selections"            : self.clipboard_selections,
                 },
             }
 
@@ -105,7 +107,7 @@ class ClipboardConnection(StubSourceMixin):
             return
         if getattr(self, "suspended", False):
             return
-        now = monotonic_time()
+        now = monotonic()
         self.clipboard_stats.append(now)
         if len(self.clipboard_stats)>=MAX_CLIPBOARD_LIMIT:
             event = self.clipboard_stats[-MAX_CLIPBOARD_LIMIT]
@@ -115,7 +117,7 @@ class ClipboardConnection(StubSourceMixin):
                 msg = "more than %s clipboard requests per second!" % MAX_CLIPBOARD_LIMIT
                 log.warn("Warning: %s", msg)
                 #disable if this rate is sustained for more than S seconds:
-                events = [x for x in self.clipboard_stats if x>(now-MAX_CLIPBOARD_LIMIT_DURATION)]
+                events = [x for x in tuple(self.clipboard_stats) if x>(now-MAX_CLIPBOARD_LIMIT_DURATION)]
                 log("%i events in the last %i seconds: %s", len(events), MAX_CLIPBOARD_LIMIT_DURATION, events)
                 if len(events)>=MAX_CLIPBOARD_LIMIT*MAX_CLIPBOARD_LIMIT_DURATION:
                     log.warn(" limit sustained for more than %i seconds,", MAX_CLIPBOARD_LIMIT_DURATION)
@@ -129,9 +131,5 @@ class ClipboardConnection(StubSourceMixin):
         packet = list(packet)
         for i, item in enumerate(packet):
             if isinstance(item, Compressible):
-                if self.brotli:
-                    packet[i] = compressed_wrapper(item.datatype, item.data,
-                                                               level=9, brotli=True, can_inline=False)
-                else:
-                    packet[i] = self.compressed_wrapper(item.datatype, item.data)
+                packet[i] = compressed_wrapper(item.datatype, item.data, level=9, can_inline=False, brotli=True)
         self.queue_packet(packet)
