@@ -366,10 +366,11 @@ class BaseWindowModel(CoreX11WindowModel):
         group_leader = None
         if "window_group" in wm_hints:
             xid = wm_hints.get("window_group")
-            try:
-                group_leader = xid, get_pywindow(xid)
-            except Exception:
-                group_leader = xid, None
+            if xid:
+                try:
+                    group_leader = get_pywindow(xid)
+                except Exception:
+                    log.error("Error locating group leader window %#x", xid)
         self._updateprop("group-leader", group_leader)
         self._updateprop("attention-requested", wm_hints.get("urgency", False))
         _input = wm_hints.get("input")
@@ -576,7 +577,7 @@ class BaseWindowModel(CoreX11WindowModel):
                     event, prop, v, STATE_STRING.get(mode, mode), current)
                 if v!=current:
                     self.update_wm_state(prop, v)
-            atom1 = get_pyatom(event.window, event.data[1])
+            atom1 = get_pyatom(event.data[1])
             log("_NET_WM_STATE: %s", atom1)
             if atom1=="_NET_WM_STATE_FULLSCREEN":
                 update_wm_state("fullscreen")
@@ -592,9 +593,9 @@ class BaseWindowModel(CoreX11WindowModel):
                 update_wm_state("skip-taskbar")
             elif atom1=="_NET_WM_STATE_SKIP_PAGER":
                 update_wm_state("skip-pager")
-                get_pyatom(event.window, event.data[2])
+                get_pyatom(event.data[2])
             elif atom1 in ("_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ"):
-                atom2 = get_pyatom(event.window, event.data[2])
+                atom2 = get_pyatom(event.data[2])
                 #we only have one state for both, so we require both to be set:
                 if atom1!=atom2 and atom2 in ("_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ"):
                     update_wm_state("maximized")
@@ -607,7 +608,7 @@ class BaseWindowModel(CoreX11WindowModel):
             elif atom1=="_NET_WM_STATE_DEMANDS_ATTENTION":
                 update_wm_state("attention-requested")
             else:
-                log.info("Unhandled _NET_WM_STATE request: '%s'", event, atom1)
+                log.info("Unhandled _NET_WM_STATE request: '%s'", atom1)
                 log.info(" event%s", event)
             return True
         if event.message_type=="WM_CHANGE_STATE":
@@ -654,12 +655,11 @@ class BaseWindowModel(CoreX11WindowModel):
             log("_NET_WM_FULLSCREEN_MONITORS: %s", event)
             #TODO: we should validate the indexes instead of copying them blindly!
             #TODO: keep track of source indication so we can forward that to the client
-            m1, m2, m3, m4 = event.data[0], event.data[1], event.data[2], event.data[3]
             N = 16      #FIXME: arbitrary limit
-            if m1<0 or m1>=N or m2<0 or m2>=N or m3<0 or m3>=N or m4<0 or m4>=N:
-                log.warn("invalid list of _NET_WM_FULLSCREEN_MONITORS - ignored")
+            monitors = list(event.data[:4])
+            if not all(0 <= x < N for x in monitors):
+                log.warn("Warning: invalid list of _NET_WM_FULLSCREEN_MONITORS:%s - ignored", event.data)
                 return False
-            monitors = [m1, m2, m3, m4]
             log("_NET_WM_FULLSCREEN_MONITORS: monitors=%s", monitors)
             prop_set(self.client_window, "_NET_WM_FULLSCREEN_MONITORS", ["u32"], monitors)
             return True
