@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -9,29 +9,33 @@
 import os
 import threading
 from time import monotonic
-from gi.repository import Gdk
+import gi
 
 from xpra.x11.bindings.core_bindings import set_context_check, X11CoreBindings     #@UnresolvedImport
 from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
-from xpra.gtk_common.error import XError, xswallow, xsync, xlog, trap, verify_sync
+from xpra.gtk_common.error import XError, xswallow, xsync, xlog, verify_sync
 from xpra.gtk_common.gtk_util import get_default_root_window
 from xpra.server.server_uuid import save_uuid, get_uuid, save_mode
-from xpra.x11.vfb_util import parse_resolution
+from xpra.x11.vfb_util import parse_resolutions
 from xpra.x11.fakeXinerama import find_libfakeXinerama, save_fakeXinerama_config, cleanup_fakeXinerama
 from xpra.x11.gtk_x11.prop import prop_get, prop_set, prop_del
 from xpra.x11.gtk_x11.gdk_display_source import close_gdk_display_source
 from xpra.x11.gtk_x11.gdk_bindings import init_x11_filter, cleanup_x11_filter, cleanup_all_event_receivers
-from xpra.common import MAX_WINDOW_SIZE
+from xpra.common import MAX_WINDOW_SIZE, FULL_INFO
 from xpra.os_util import strtobytes
 from xpra.util import typedict, envbool, first_time, XPRA_DPI_NOTIFICATION_ID
 from xpra.net.compression import Compressed
 from xpra.server.gtk_server_base import GTKServerBase
+from xpra.server import server_features
 from xpra.x11.xkbhelper import clean_keyboard_state
 from xpra.scripts.config import FALSE_OPTIONS
 from xpra.log import Logger
 
+gi.require_version('Gdk', '3.0')
+from gi.repository import Gdk  # pylint: disable=wrong-import-position, wrong-import-order
+
 set_context_check(verify_sync)
 RandR = RandRBindings()
 X11Keyboard = X11KeyboardBindings()
@@ -50,6 +54,7 @@ xinputlog = Logger("xinput")
 
 ALWAYS_NOTIFY_MOTION = envbool("XPRA_ALWAYS_NOTIFY_MOTION", False)
 FAKE_X11_INIT_ERROR = envbool("XPRA_FAKE_X11_INIT_ERROR", False)
+DUMMY_WIDTH_HEIGHT_MM = envbool("XPRA_DUMMY_WIDTH_HEIGHT_MM", True)
 
 
 class XTestPointerDevice:
@@ -79,7 +84,6 @@ class X11ServerCore(GTKServerBase):
     """
 
     def __init__(self):
-        self.screen_number = Gdk.Screen.get_default().get_number()
         self.root_window = get_default_root_window()
         self.pointer_device = XTestPointerDevice()
         self.touchpad_device = None
@@ -98,7 +102,6 @@ class X11ServerCore(GTKServerBase):
 
     def server_init(self):
         self.x11_init()
-        from xpra.server import server_features
         if server_features.windows:
             from xpra.x11.x11_window_filters import init_x11_window_filters
             init_x11_window_filters()
@@ -106,10 +109,10 @@ class X11ServerCore(GTKServerBase):
 
     def do_init(self, opts):
         try:
-            self.initial_resolution = parse_resolution(opts.resize_display)
+            self.initial_resolutions = parse_resolutions(opts.resize_display, opts.refresh_rate)
         except ValueError:
-            pass
-        self.randr = bool(self.initial_resolution) or not (opts.resize_display in FALSE_OPTIONS)
+            self.initial_resolutions = None
+        self.randr = opts.resize_display not in FALSE_OPTIONS
         self.randr_exact_size = False
         self.fake_xinerama = "no"      #only enabled in seamless server
         self.current_xinerama_config = None
@@ -162,7 +165,7 @@ class X11ServerCore(GTKServerBase):
         #in which case the screen sizes list may be longer than 1
         eprop = prop_get(self.root_window, "_XPRA_RANDR_EXACT_SIZE", "u32", ignore_errors=True, raise_xerrors=False)
         screenlog("_XPRA_RANDR_EXACT_SIZE=%s", eprop)
-        self.randr_exact_size = eprop==1
+        self.randr_exact_size = eprop==1 or RandR.get_version()>=(1, 6)
         if not self.randr_exact_size:
             #ugly hackish way of detecting Xvfb with randr,
             #assume that it has only one resolution pre-defined:
@@ -189,8 +192,9 @@ class X11ServerCore(GTKServerBase):
         def get_default_cursor():
             self.default_cursor_image = X11Keyboard.get_cursor_image()
             cursorlog("get_default_cursor=%s", self.default_cursor_image)
-        trap.swallow_synced(get_default_cursor)
-        X11Keyboard.selectCursorChange(True)
+        with xlog:
+            get_default_cursor()
+            X11Keyboard.selectCursorChange(True)
 
     def get_display_bit_depth(self):
         with xlog:
@@ -223,31 +227,31 @@ class X11ServerCore(GTKServerBase):
 
 
     def set_keyboard_layout_group(self, grp):
-        if not self.keyboard_config:
-            keylog("set_keyboard_layout_group(%i) ignored, no config", grp)
+        kc = self.keyboard_config
+        if not kc:
+            keylog(f"set_keyboard_layout_group({grp}) ignored, no config")
             return
-        if not self.keyboard_config.xkbmap_layout_groups:
-            keylog("set_keyboard_layout_group(%i) ignored, no layout groups support", grp)
+        if not kc.layout_groups:
+            keylog(f"set_keyboard_layout_group({grp}) ignored, no layout groups support")
             #not supported by the client that owns the current keyboard config,
             #so make sure we stick to the default group:
             grp = 0
         if not X11Keyboard.hasXkb():
-            keylog("set_keyboard_layout_group(%i) ignored, no Xkb support", grp)
+            keylog(f"set_keyboard_layout_group({grp}) ignored, no Xkb support")
             return
         if grp<0:
             grp = 0
         if self.current_keyboard_group!=grp:
-            keylog("set_keyboard_layout_group(%i) ignored, value unchanged", grp)
+            keylog(f"set_keyboard_layout_group({grp}) ignored, value unchanged")
             return
-        keylog("set_keyboard_layout_group(%i) config=%s, current keyboard group=%s",
-               grp, self.keyboard_config, self.current_keyboard_group)
+        keylog(f"set_keyboard_layout_group({grp}) config={self.keyboard_config}, current keyboard group={self.current_keyboard_group}")
         try:
             with xsync:
                 self.current_keyboard_group = X11Keyboard.set_layout_group(grp)
         except XError as e:
-            keylog("set_keyboard_layout_group group=%s", grp, exc_info=True)
-            keylog.error("Error: failed to set keyboard layout group '%s'", grp)
-            keylog.error(" %s", e)
+            keylog(f"set_keyboard_layout_group({grp})", exc_info=True)
+            keylog.error(f"Error: failed to set keyboard layout group {grp}")
+            keylog.estr(e)
 
     def init_packet_handlers(self):
         super().init_packet_handlers()
@@ -314,24 +318,25 @@ class X11ServerCore(GTKServerBase):
         save_uuid(str(self.uuid))
 
     def set_keyboard_repeat(self, key_repeat):
-        if key_repeat:
-            self.key_repeat_delay, self.key_repeat_interval = key_repeat
-            if self.key_repeat_delay>0 and self.key_repeat_interval>0:
-                X11Keyboard.set_key_repeat_rate(self.key_repeat_delay, self.key_repeat_interval)
-                keylog.info("setting key repeat rate from client: %sms delay / %sms interval",
-                            self.key_repeat_delay, self.key_repeat_interval)
-        else:
-            #dont do any jitter compensation:
-            self.key_repeat_delay = -1
-            self.key_repeat_interval = -1
-            #but do set a default repeat rate:
-            X11Keyboard.set_key_repeat_rate(500, 30)
-            keylog("keyboard repeat disabled")
+        with xlog:
+            if key_repeat:
+                self.key_repeat_delay, self.key_repeat_interval = key_repeat
+                if self.key_repeat_delay>0 and self.key_repeat_interval>0:
+                    X11Keyboard.set_key_repeat_rate(self.key_repeat_delay, self.key_repeat_interval)
+                    keylog.info("setting key repeat rate from client: %sms delay / %sms interval",
+                                self.key_repeat_delay, self.key_repeat_interval)
+            else:
+                #dont do any jitter compensation:
+                self.key_repeat_delay = -1
+                self.key_repeat_interval = -1
+                #but do set a default repeat rate:
+                X11Keyboard.set_key_repeat_rate(500, 30)
+                keylog("keyboard repeat disabled")
 
     def make_hello(self, source):
         capabilities = super().make_hello(source)
         capabilities["server_type"] = "Python/gtk/x11"
-        if source.wants_features:
+        if "features" in source.wants:
             capabilities.update({
                     "resize_screen"             : self.randr,
                     "resize_exact"              : self.randr_exact_size,
@@ -345,7 +350,7 @@ class X11ServerCore(GTKServerBase):
                 sizes = self.get_all_screen_sizes()
                 if len(sizes)>1:
                     capabilities["screen-sizes"] = sizes
-            if self.default_cursor_image and source.wants_default_cursor:
+            if self.default_cursor_image and "default_cursor" in source.wants:
                 capabilities["cursor.default"] = self.default_cursor_image
         return capabilities
 
@@ -358,6 +363,13 @@ class X11ServerCore(GTKServerBase):
             "fakeXinerama"          : bool(self.libfakeXinerama_so),
             "libfakeXinerama"       : self.libfakeXinerama_so or "",
             })
+        if FULL_INFO>1:
+            try:
+                from xpra.codecs.drm.drm import query  # pylint: disable=import-outside-toplevel
+            except ImportError as e:
+                log(f"no drm query: {e}")
+            else:
+                sinfo["drm"] = query()
         log("X11ServerCore.do_get_info took %ims", (monotonic()-start)*1000)
         return info
 
@@ -425,14 +437,24 @@ class X11ServerCore(GTKServerBase):
         return info
 
 
+    def get_window_id(self, gdkwindow):
+        if not gdkwindow:
+            return 0
+        xid = gdkwindow.get_xid()
+        if not xid:
+            return 0
+        for wid, window in self._id_to_window.items():
+            if window.get("xid", 0)==xid:
+                return wid
+        return 0
+
+
     def get_keyboard_config(self, props=typedict()):
         from xpra.x11.server_keyboard_config import KeyboardConfig
         keyboard_config = KeyboardConfig()
         keyboard_config.enabled = props.boolget("keyboard", True)
         keyboard_config.parse_options(props)
-        keyboard_config.xkbmap_layout = props.strget("xkbmap_layout")
-        keyboard_config.xkbmap_variant = props.strget("xkbmap_variant")
-        keyboard_config.xkbmap_options = props.strget("xkbmap_options")
+        keyboard_config.parse_layout(props)
         keylog("get_keyboard_config(..)=%s", keyboard_config)
         return keyboard_config
 
@@ -488,16 +510,17 @@ class X11ServerCore(GTKServerBase):
         with xlog:
             return X11Keyboard.get_cursor_image()
 
-    def get_cursor_data(self):
+    def get_cursor_data(self, skip_default=True):
         #must be called from the UI thread!
         cursor_image = self.get_cursor_image()
         if cursor_image is None:
             cursorlog("get_cursor_data() failed to get cursor image")
             return None, []
-        self.last_cursor_image = cursor_image
+        self.last_cursor_image = list(cursor_image)
         pixels = self.last_cursor_image[7]
         cursorlog("get_cursor_image() cursor=%s", cursor_image[:7]+["%s bytes" % len(pixels)]+cursor_image[8:])
-        if self.default_cursor_image is not None and str(pixels)==str(self.default_cursor_image[7]):
+        is_default = self.default_cursor_image is not None and str(pixels)==str(self.default_cursor_image[7])
+        if skip_default and is_default:
             cursorlog("get_cursor_data(): default cursor - clearing it")
             cursor_image = None
         cursor_sizes = self.get_cursor_sizes()
@@ -584,15 +607,14 @@ class X11ServerCore(GTKServerBase):
         if self.randr_exact_size:
             try:
                 with xsync:
-                    v = RandR.add_screen_size(desired_w, desired_h)
-                    if v:
+                    if RandR.add_screen_size(desired_w, desired_h):
                         #we have to wait a little bit
                         #to make sure that everything sees the new resolution
                         #(ideally this method would be split in two and this would be a callback)
-                        self.randr_sizes_added.append(v)
+                        self.randr_sizes_added.append((desired_w, desired_h))
                         import time
                         time.sleep(0.5)
-                        return v
+                        return desired_w, desired_h
             except XError as e:
                 screenlog("add_screen_size(%s, %s)", desired_w, desired_h, exc_info=True)
                 screenlog.warn("Warning: failed to add resolution %ix%i:", desired_w, desired_h)
@@ -638,6 +660,7 @@ class X11ServerCore(GTKServerBase):
         ydpi = self.ydpi or self.dpi
         screenlog("set_screen_size(%s, %s, %s) xdpi=%s, ydpi=%s",
                   desired_w, desired_h, bigger, xdpi, ydpi)
+        wmm, hmm = 0, 0
         if xdpi<=0 or ydpi<=0:
             #use some sane defaults: either the command line option, or fallback to 96
             #(96 is better than nothing, because we do want to set the dpi
@@ -646,7 +669,6 @@ class X11ServerCore(GTKServerBase):
             ydpi = self.default_dpi or 96
             #find the "physical" screen dimensions, so we can calculate the required dpi
             #(and do this before changing the resolution)
-            wmm, hmm = 0, 0
             client_w, client_h = 0, 0
             sss = self._server_sources.values()
             for ss in sss:
@@ -665,6 +687,16 @@ class X11ServerCore(GTKServerBase):
                 ydpi = round(client_h * 25.4 / hmm)
                 screenlog("calculated DPI: %s x %s (from w: %s / %s, h: %s / %s)",
                           xdpi, ydpi, client_w, wmm, client_h, hmm)
+        if wmm==0 or hmm==0:
+            wmm = round(desired_w * 25.4 / xdpi)
+            hmm = round(desired_h * 25.4 / ydpi)
+        if DUMMY_WIDTH_HEIGHT_MM:
+            #FIXME: we assume there is only one output:
+            output = 0
+            with xsync:
+                RandR.set_output_int_property(output, "WIDTH_MM", wmm)
+                RandR.set_output_int_property(output, "HEIGHT_MM", hmm)
+        screenlog("set_dpi(%i, %i)", xdpi, ydpi)
         self.set_dpi(xdpi, ydpi)
 
         #try to find the best screen size to resize to:
@@ -680,6 +712,10 @@ class X11ServerCore(GTKServerBase):
         else:
             screenlog("fakeXinerama can only be enabled for a single client (found %s)" % len(ui_clients))
         xinerama_changed = save_fakeXinerama_config(self.fake_xinerama and len(ui_clients)==1, source, screen_sizes)
+        with xsync:
+            if RandR.is_dummy16():
+                #don't bother - randr16 does it better
+                xinerama_changed = False
         #we can only keep things unchanged if xinerama was also unchanged
         #(many apps will only query xinerama again if they get a randr notification)
         if (w==root_w and h==root_h) and not xinerama_changed:
@@ -717,7 +753,7 @@ class X11ServerCore(GTKServerBase):
                 #we can use XRRSetScreenSize:
                 try:
                     with xsync:
-                        RandR.xrr_set_screen_size(w, h, self.xdpi or self.dpi or 96, self.ydpi or self.dpi or 96)
+                        RandR.xrr_set_screen_size(w, h, wmm, hmm)
                 except XError:
                     screenlog("XRRSetScreenSize failed", exc_info=True)
             screenlog("calling RandR.get_screen_size()")
@@ -841,7 +877,8 @@ class X11ServerCore(GTKServerBase):
         wid = 0
         for ss in self.window_sources():
             name = strtobytes(event.bell_name or "")
-            ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
+            ss.bell(wid, event.device, event.percent,
+                    event.pitch, event.duration, event.bell_class, event.bell_id, name)
 
 
     def _bell_signaled(self, wm, event):
@@ -854,20 +891,11 @@ class X11ServerCore(GTKServerBase):
         log("_bell_signaled(%s,%r) wid=%s", wm, event, wid)
         for ss in self.window_sources():
             name = strtobytes(event.bell_name or "")
-            ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, name)
-
-
-    def get_screen_number(self, _wid):
-        #maybe this should be in all cases (it is in desktop_server):
-        #model = self._id_to_window.get(wid)
-        #return model.client_window.get_screen().get_number()
-        #return Gdk.Display.get_default().get_default_screen().get_number()
-        #-1 uses the current screen
-        return -1
+            ss.bell(wid, event.device, event.percent,
+                    event.pitch, event.duration, event.bell_class, event.bell_id, name)
 
 
     def setup_input_devices(self):
-        from xpra.server import server_features
         xinputlog("setup_input_devices() input_devices feature=%s", server_features.input_devices)
         if not server_features.input_devices:
             return
@@ -914,6 +942,7 @@ class X11ServerCore(GTKServerBase):
         if not ss:
             return
         wid, button, distance, pointer, modifiers, _buttons = packet[1:7]
+        self.record_wheel_event(wid, button)
         with xsync:
             if self.do_process_mouse_common(proto, wid, pointer):
                 self.last_mouse_user = ss.uuid
@@ -942,14 +971,18 @@ class X11ServerCore(GTKServerBase):
                 geom = model.get_geometry()
                 x = geom[0]+rx
                 y = geom[1]+ry
-                log("_get_pointer_abs_coordinates(%i, %s)=%s window geometry=%s", wid, pos, (x, y), geom)
+                mouselog("_get_pointer_abs_coordinates(%i, %s)=%s window geometry=%s", wid, pos, (x, y), geom)
         return x, y
 
     def _move_pointer(self, wid, pos, deviceid=-1, *args):
         #(this is called within an xswallow context)
-        screen_no = self.get_screen_number(wid)
-        device = self.get_pointer_device(deviceid)
         x, y = self._get_pointer_abs_coordinates(wid, pos)
+        self.device_move_pointer(wid, (x, y), deviceid)
+
+    def device_move_pointer(self, wid, pos, deviceid=-1, *args):
+        screen_no = -1
+        device = self.get_pointer_device(deviceid)
+        x, y = pos
         mouselog("move_pointer(%s, %s, %s) screen_no=%i, device=%s, position=%s",
                  wid, pos, deviceid, screen_no, device, (x, y))
         try:
@@ -979,25 +1012,31 @@ class X11ServerCore(GTKServerBase):
             if wid==self.get_focus():
                 ss.user_event()
 
-    def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, _buttons=(), deviceid=-1, *_args):
+    def do_process_button_action(self, proto, wid, button, pressed, pointer,
+                                 modifiers, _buttons=(), deviceid=-1, *_args):
         self._update_modifiers(proto, wid, modifiers)
         #TODO: pass extra args
         if self._process_mouse_common(proto, wid, pointer, deviceid):
-            self.button_action(pointer, button, pressed, deviceid)
+            self.button_action(wid, pointer, button, pressed, deviceid)
 
-    def button_action(self, pointer, button, pressed, deviceid=-1, *args):
+    def button_action(self, wid, pointer, button, pressed, deviceid=-1, *args):
         device = self.get_pointer_device(deviceid)
         assert device, "pointer device %s not found" % deviceid
+        if button in (4, 5) and wid:
+            self.record_wheel_event(wid, button)
         try:
-            log("%s%s", device.click, (button, pressed, args))
+            mouselog("%s%s", device.click, (button, pressed, args))
             with xsync:
                 device.click(button, pressed, *args)
         except XError:
-            log("button_action(%s, %s, %s, %s, %s)", pointer, button, pressed, deviceid, args, exc_info=True)
-            log.error("Error: failed (un)press mouse button %s", button)
-            if button>=4:
-                log.error(" (perhaps your Xvfb does not support mousewheels?)")
+            mouselog("button_action(%s, %s, %s, %s, %s, %s)",
+                     wid, pointer, button, pressed, deviceid, args, exc_info=True)
+            mouselog.error("Error: failed (un)press mouse button %s", button)
 
+    def record_wheel_event(self, wid, button):
+        mouselog("recording scroll event for button %i", button)
+        for ss in self.window_sources():
+            ss.record_scroll_event(wid)
 
     def make_screenshot_packet_from_regions(self, regions):
         #regions = array of (wid, x, y, PIL.Image)
@@ -1012,7 +1051,7 @@ class X11ServerCore(GTKServerBase):
         width = maxx-minx
         height = maxy-miny
         log("screenshot: %sx%s, min x=%s y=%s", width, height, minx, miny)
-        from PIL import Image                           #@UnresolvedImport
+        from PIL import Image                           #@UnresolvedImport pylint: disable=import-outside-toplevel
         screenshot = Image.new("RGBA", (width, height))
         for wid, x, y, img in reversed(regions):
             pixel_format = img.get_pixel_format()
