@@ -1,16 +1,15 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
-import re
 
 #ensure that we use gtk as display source:
 from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
-from xpra.util import std, csv
+from xpra.util import std, csv, envbool, typedict
 from xpra.os_util import bytestostr
-from xpra.gtk_common.error import xsync
+from xpra.gtk_common.error import xsync, xlog
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 from xpra.log import Logger
 
@@ -19,6 +18,7 @@ X11Keyboard = X11KeyboardBindings()
 
 log = Logger("x11", "keyboard")
 
+XKB = envbool("XPRA_XKB", True)
 DEBUG_KEYSYMS = [x for x in os.environ.get("XPRA_DEBUG_KEYSYMS", "").split(",") if len(x)>0]
 
 #keys we choose not to map if the free space in the keymap is too limited
@@ -47,81 +47,63 @@ OPTIONAL_KEYS = [
 
 
 def clean_keyboard_state():
-    try:
+    with xlog:
         X11Keyboard.ungrab_all_keys()
-    except:
-        log.error("error ungrabbing keys", exc_info=True)
-    try:
+    with xlog:
+        X11Keyboard.set_layout_group(0)
+    with xlog:
         X11Keyboard.unpress_all_keys()
-    except:
-        log.error("error unpressing keys", exc_info=True)
 
 ################################################################################
 # keyboard layouts
 
-def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
-                  xkbmap_print, xkbmap_query, xkbmap_query_struct={}):
-    """ xkbmap_layout is the generic layout name (used on non posix platforms)
-        xkbmap_variant is the layout variant (may not be set)
-        xkbmap_print is the output of "setxkbmap -print" on the client
-        xkbmap_query is the output of "setxkbmap -query" on the client
-        xkbmap_query_struct is xkbmap_query parsed into a dictionary
+def do_set_keymap(layout, variant, options, query_struct):
+    """
+        layout is the generic layout name (used on non posix platforms)
+            defaults to `us`
+        variant is the layout variant (optional)
+        query_struct is the output of "setxkbmap -query" on the client
+        parsed into a dictionary
         Use those to try to setup the correct keyboard map for the client
         so that all the keycodes sent will be mapped
     """
     #First we try to use data from setxkbmap -query,
     #preferably as structured data:
-    if xkbmap_query and not xkbmap_query_struct:
-        from xpra.keyboard.layouts import parse_xkbmap_query
-        xkbmap_query_struct = parse_xkbmap_query(xkbmap_query)
-    if xkbmap_query_struct:
-        log("do_set_keymap using xkbmap_query struct=%s", xkbmap_query_struct)
-        """ The xkbmap_query_struct data will look something like this:
-            {
-            b"rules"       : b"evdev",
-            b"model"       : b"pc105",
-            b"layout"      : b"gb",
-            b"options"     : b"grp:shift_caps_toggle",
-            }
-        """
+    query_struct = typedict(query_struct)
+    if query_struct:
+        log("do_set_keymap using xkbmap_query struct=%s", query_struct)
+        #The query_struct data will look something like this:
+        #    {
+        #    b"rules"       : b"evdev",
+        #    b"model"       : b"pc105",
+        #    b"layout"      : b"gb",
+        #    b"options"     : b"grp:shift_caps_toggle",
+        #    }
         #parse the data into a dict:
-        rules = xkbmap_query_struct.get(b"rules")
-        model = xkbmap_query_struct.get(b"model")
-        layout = xkbmap_query_struct.get(b"layout")
-        variant = xkbmap_query_struct.get(b"variant")
-        options = xkbmap_query_struct.get(b"options")
+        rules = query_struct.strget("rules")
+        model = query_struct.strget("model")
+        layout = query_struct.strget("layout")
+        variant = query_struct.strget("variant")
+        options = query_struct.strget("options")
         if layout:
             log.info("setting keymap: %s",
-                     csv("%s=%s" % (std(k), std(v)) for k,v in xkbmap_query_struct.items()
-                         if k in ("rules", "model", "layout") and v))
-            safe_setxkbmap(rules, model, layout, variant, options)
+                     csv("%s=%s" % (std(k), std(v)) for k,v in query_struct.items()
+                         if k in ("rules", "model", "layout", "variant", "options") and v))
+            if safe_setxkbmap(rules, model, layout, variant, options):
+                return
         else:
-            safe_setxkbmap(rules, model, "", "", "")
-    if xkbmap_print:
-        #TODO: this is no longer used with any client, remove it
-        log("do_set_keymap using xkbmap_print")
-        #try to guess the layout by parsing "setxkbmap -print"
-        try:
-            sym_re = re.compile(r"\s*xkb_symbols\s*{\s*include\s*\"([\w\+]*)")
-            for line in xkbmap_print.splitlines():
-                m = sym_re.match(line)
-                if m:
-                    layout = std(m.group(1))
-                    log.info("guessing keyboard layout='%s'" % layout)
-                    safe_setxkbmap("evdev", "pc105", layout, "", xkbmap_options)
-                    return
-        except Exception as e:
-            log.info("error setting keymap: %s" % e)
+            if safe_setxkbmap(rules, model, "", "", ""):
+                return
     #fallback for non X11 clients:
-    layout = xkbmap_layout or "us"
+    layout = layout or "us"
     log.info("setting keyboard layout to '%s'", std(layout))
-    safe_setxkbmap("evdev", "pc105", layout, xkbmap_variant, xkbmap_options)
+    safe_setxkbmap("evdev", "pc105", layout, variant, options)
 
 def safe_setxkbmap(rules, model, layout, variant, options):
     #(we execute the options separately in case that fails..)
     try:
         X11Keyboard.setxkbmap(rules, model, layout, variant, options)
-        return
+        return True
     except Exception:
         log("safe_setxkbmap%s", (rules, model, layout, variant, options), exc_info=True)
         log.warn("Warning: failed to set exact keymap,")
@@ -131,11 +113,12 @@ def safe_setxkbmap(rules, model, layout, variant, options):
         #try again with no options:
         try:
             X11Keyboard.setxkbmap(rules, model, layout, variant, "")
+            return True
         except Exception:
             log("setxkbmap", exc_info=True)
             log.error("Error: failed to set exact keymap")
             log.error(" even without applying any options..")
-
+    return False
 
 ################################################################################
 # keycodes
@@ -144,8 +127,9 @@ def apply_xmodmap(instructions):
     try:
         with xsync:
             unset = X11Keyboard.set_xmodmap(instructions)
-    except:
-        log.error("Error configuring modifier map", exc_info=True)
+    except Exception as e:
+        log("apply_xmodmap(%s)", instructions, exc_info=True)
+        log.error("Error configuring modifier map: %s", e)
         unset = instructions
     if unset is None:
         #None means an X11 error occurred, re-do all:
@@ -153,6 +137,11 @@ def apply_xmodmap(instructions):
     return unset
 
 
+def get_keycode_mappings():
+    if XKB and X11Keyboard.hasXkb():
+        return X11Keyboard.get_xkb_keycode_mappings()
+    return X11Keyboard.get_keycode_mappings()
+
 def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
     """
         Translate the given keycodes into the existing keymap
@@ -168,7 +157,7 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
     #keycodes = {
     #     9: set([('', 1), ('Escape', 4), ('', 3), ('Escape', 0), ('Escape', 2)]),
     #     10: set([('onesuperior', 4), ('onesuperior', 8), ('exclam', 1), ('1', 6), ('exclam', 3), ('1', 2), ('exclamdown', 9), ('exclamdown', 5), ('1', 0), ('exclam', 7)]),
-    x11_keycodes = X11Keyboard.get_keycode_mappings()
+    x11_keycodes = get_keycode_mappings()
     log(" x11_keycodes=%s", x11_keycodes)
     #x11_keycodes = {
     #    8: ['Mode_switch', '', 'Mode_switch', '', 'Mode_switch'],
@@ -182,35 +171,41 @@ def set_keycode_translation(xkbmap_x11_keycodes, xkbmap_keycodes):
         keycodes = tuple(x11_keycodes_for_keysym.get(keysym, set()))
         if keysym in DEBUG_KEYSYMS:
             log.info("set_keycode_translation: find_keycode%s x11 keycodes=%s", (kc, keysym, i), keycodes)
-        def rlog(v, msg):
+        def rlog(keycode, msg):
             if keysym in DEBUG_KEYSYMS:
-                log.info("set_keycode_translation: find_keycode%s=%s (%s)", (kc, keysym, i), v, msg)
-            return v
+                log.info("set_keycode_translation: find_keycode%s=%s (%s)", (kc, keysym, i), keycode, msg)
         if not keycodes:
             return None
         #no other option, use it:
-        if len(keycodes)==1:
-            return keycodes[0]
         for keycode in keycodes:
             defs = x11_keycodes.get(keycode)
             if keysym in DEBUG_KEYSYMS:
-                log.info("x11 keycode %i: %s", keycode, defs)
+                log.info("server x11 keycode %i: %s", keycode, defs)
             assert defs, "bug: keycode %i not found in %s" % (keycode, x11_keycodes)
             if len(defs)>i and defs[i]==keysym:
-                return rlog(keycode, "exact index match")
+                rlog(keycode, "exact index match")
+                return keycode, True
         #if possible, use the same one:
         if kc in keycodes:
-            return rlog(kc, "using same keycode as client")
-        return rlog(keycodes[0], "using first match")
+            rlog(kc, "using same keycode as client")
+            return kc, False
+        keycode = keycodes[0]
+        rlog(keycode, "using first match")
+        return keycode, False
     #generate the translation map:
     trans = {}
     for keycode, defs in keycodes.items():
-        for keysym,i in tuple(defs):             #ie: ('1', 0) or ('A', 1), etc
-            x11_keycode = find_keycode(keycode, keysym, i)
-            if x11_keycode:
+        if bool(set(DEBUG_KEYSYMS) & set(bytestostr(d[0]) for d in defs)):
+            log.info("client keycode=%i, defs=%s", keycode, defs)
+        for bkeysym, i in tuple(defs):             #ie: (b'1', 0) or (b'A', 1), etc
+            keysym = bytestostr(bkeysym)
+            m = find_keycode(keycode, keysym, i)
+            if m:
+                x11_keycode, index_matched = m
                 trans[(keycode, keysym)] = x11_keycode
-                trans[(keysym, i)] = x11_keycode
                 trans[keysym] = x11_keycode
+                if index_matched:
+                    trans[(keysym, i)] = x11_keycode
     if not xkbmap_x11_keycodes:
         #now add all the keycodes we may not have mapped yet
         #(present in x11_keycodes but not keycodes)
@@ -278,7 +273,7 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
         def estr(entries):
             try:
                 return csv(tuple(set(x[0] for x in entries)))
-            except:
+            except Exception:
                 return csv(tuple(entries))
         for keycode, entries in mappings.items():
             mods = modifiers_for(entries)
@@ -339,7 +334,10 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
     for try_harder in (False, True):
         filtered_preserve_keycode_entries = filter_mappings(indexed_mappings(preserve_keycode_entries), try_harder)
         log("filtered_preserve_keycode_entries=%s", filtered_preserve_keycode_entries)
-        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax, keycodes, filtered_preserve_keycode_entries, keysym_to_modifier, try_harder)
+        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax, keycodes,
+                                                                   filtered_preserve_keycode_entries,
+                                                                   keysym_to_modifier,
+                                                                   try_harder)
         if not missing_keycodes:
             break
     instructions = keymap_to_xmodmap(new_keycodes)
@@ -412,7 +410,7 @@ def x11_keycodes_to_list(x11_mappings):
     return entries
 
 
-def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keysym_to_modifier={}, try_harder=False):
+def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries, keysym_to_modifier, try_harder=False):
     """
         The keycodes given may not match the range that the server supports,
         or some of those keycodes may not be usable (only one modifier can
@@ -608,7 +606,7 @@ def keymap_to_xmodmap(trans_keycodes):
             assert 0<=index<keysyms_per_keycode
             try:
                 keysym = X11Keyboard.parse_keysym(name)
-            except:
+            except Exception:
                 keysym = None
             if keysym is None:
                 if name!="":
@@ -657,10 +655,8 @@ def keymap_to_xmodmap(trans_keycodes):
 ################################################################################
 # modifiers
 
-def clear_modifiers(_modifiers):
-    instructions = []
-    for i in range(0, 8):
-        instructions.append(("clear", i))
+def clear_modifiers():
+    instructions = [("clear", i) for i in range(0, 8)]
     apply_xmodmap(instructions)
 
 def set_modifiers(modifiers):
@@ -725,8 +721,9 @@ def get_modifiers_from_keycodes(xkbmap_keycodes, add_default_modifiers=True):
         _, keyname, _, _, _ = entry
         modifier = pref.get(keyname)
         if modifier:
-            keynames = matches.setdefault(modifier, set())
-            keynames.add(keyname)
+            keynames = matches.setdefault(modifier, [])
+            if keyname not in keynames:
+                keynames.append(keyname)
             all_keynames.add(keyname)
     if add_default_modifiers:
         #try to add missings ones (magic!)
@@ -736,10 +733,14 @@ def get_modifiers_from_keycodes(xkbmap_keycodes, add_default_modifiers=True):
                 continue            #aleady defined
             if modifier not in matches:
                 #define it since it is completely missing
-                defaults.setdefault(modifier, set()).add(keyname)
+                keynames = defaults.setdefault(modifier, [])
+                if keyname not in keynames:
+                    keynames.append(keyname)
             elif modifier in ["shift", "lock", "control", "mod1", "mod2"] or keyname=="ISO_Level3_Shift":
                 #these ones we always add them, even if a record for this modifier already exists
-                matches.setdefault(modifier, set()).add(keyname)
+                keynames = matches.setdefault(modifier, [])
+                if keyname not in keynames:
+                    keynames.append(keyname)
         log("get_modifiers_from_keycodes(...) adding defaults: %s", defaults)
         matches.update(defaults)
     log("get_modifiers_from_keycodes(...)=%s", matches)
